@@ -12,6 +12,7 @@ import re
 import hashlib
 import base64
 import json
+import random
 
 # Example :
 #   ./monitor_elections.py --uuid aTGmQNj1SXA5JG --url https://belenios.loria.fr/ --wdir /tmp/wdir --checkhash yes --hashref $HOME/hashref --outputref  $HOME/hashref --sighashref https://belenios.loria.fr/monitoring-reference/reference.json.gpg --keyring $HOME/.gnupg/pubring.gpg
@@ -24,7 +25,7 @@ import json
 
 
 # TODO:
-# - add options --belenios-tool-path 
+# - add options --belenios-tool-path
 # - find a way to test that failure are detected
 # - do a git gc from time to time or at the end (how?)
 
@@ -57,6 +58,8 @@ def Elogme(str):
     logme(str)
     print("Log: {}".format(str), file=sys.stderr)
 
+def b64_of_hex(s):
+    return base64.b64encode(bytes.fromhex(s)).decode().strip("=")
 
 # If it does not exist, create a fresh directory for an election
 # and initialize the git.
@@ -80,13 +83,18 @@ audit_files=['election.json', 'public_creds.txt', 'trustees.json',
 optional_audit_files=['ballots.jsons','result.json','shuffles.jsons',
         'hash_voterlist','all_ballot_hashs']
 
+def shuffle(l):
+    result = [x for x in l]
+    random.shuffle(result)
+    return result
+
 def download_audit_data(url, uuid):
     link = url + '/elections/' + uuid
     data = dict()
     status = Status(False, b"")
     fail = False
     msg = ""
-    for f in audit_files:
+    for f in shuffle(audit_files):
         try:
             if f == 'index.html':
                 l = link + '/'
@@ -97,7 +105,7 @@ def download_audit_data(url, uuid):
         except urllib.error.URLError as e:
             fail = True
             msg = msg + "Download {} failed with ret code \"{}\" for election {}\n".format(f, e, uuid)
-    for f in optional_audit_files:
+    for f in shuffle(optional_audit_files):
         try:
             resp = urllib.request.urlopen(link + '/' + f)
             data[f]=resp.read()
@@ -108,24 +116,10 @@ def download_audit_data(url, uuid):
     return status, data
 
 def get_new_ballots(old_ballotsfile, new_ballotsfile):
-    old = set()
-    if os.path.exists(old_ballotsfile):
-        with open(old_ballotsfile, "r") as f:
-            ll = f.read()
-        for l in ll.splitlines():
-            m = hashlib.sha256()
-            m.update(l.encode())
-            h = base64.b64encode(m.digest()).decode().strip('=')
-            old.add(h)
-    ll = ""
-    if os.path.exists(new_ballotsfile):
-        with open(new_ballotsfile, "r") as f:
-            ll = f.read()
+    old = set([b64_of_hex(x["hash"]) for x in json.loads(old_ballotsfile)])
     result = b""
-    for l in ll.splitlines():
-        m = hashlib.sha256()
-        m.update(l.encode())
-        h = base64.b64encode(m.digest()).decode().strip('=')
+    for x in json.loads(new_ballotsfile):
+        h = b64_of_hex(x["hash"])
         if not h in old:
             result = result + h.encode() + b"\n"
     return result
@@ -144,32 +138,62 @@ def write_and_verify_new_data(wdir, uuid, data):
                 newf.write(data[f])
 
     # run belenios-tool verify on it
-    ver = subprocess.run(["belenios-tool", "verify", "--dir={}".format(pnew)],
+    ver = subprocess.run(["belenios-tool", "election", "verify", "--dir={}".format(pnew)],
             stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
     if ver.returncode != 0:
-        msg="Error: belenios-tool verify failed on newly downloaded data from election {}, with output {}\n".format(uuid, ver.stdout).encode()
+        msg="Error: belenios-tool election verify failed on newly downloaded data from election {}, with output {}\n".format(uuid, ver.stdout).encode()
         return Status(True, msg)
     else:
         logme("Successfully verified new data of {}".format(uuid))
 
-    # if not the first time, run belenios-tool verify-diff
+    # if not the first time, run belenios-tool election verify-diff
     msg = b""
     new_ballots = b""
-    if os.path.exists(os.path.join(p, "fresh")):
+    fresh = os.path.exists(os.path.join(p, "fresh"))
+    if fresh:
         os.remove(os.path.join(p, "fresh"))
     else:
-        verdiff = subprocess.run(["belenios-tool", "verify-diff",
+        verdiff = subprocess.run(["belenios-tool", "election", "verify-diff",
             "--dir1={}".format(p), "--dir2={}".format(pnew)],
             stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
         if verdiff.returncode != 0:
-            msg="Error: belenios-tool verify-diff failed on newly downloaded data from election {}, with output {}".format(uuid, verdiff.stdout).encode()
+            msg="Error: belenios-tool election verify-diff failed on newly downloaded data from election {}, with output {}".format(uuid, verdiff.stdout).encode()
             return Status(True, msg)
         if re.search(b"W:", verdiff.stdout) != None:
             msg = verdiff.stdout
         logme("Successfully diff-verified new data of {}".format(uuid))
-    new_ballots=get_new_ballots(os.path.join(p, "ballots.jsons"),
-            os.path.join(pnew, "ballots.jsons"))
+
+    # ballots of old data
+    if fresh:
+        ballot_summary1 = "[]"
+    else:
+        ballot_summary1 = subprocess.run(["belenios-tool", "election", "compute-ballot-summary", "--dir={}".format(p)],
+                                         stdout=subprocess.PIPE, stderr=subprocess.DEVNULL)
+        if ballot_summary1.returncode != 0:
+            msg = "Error: compute-ballot-summary on old data failed for election {}".format(uuid).encode()
+            return Status(True, msg)
+        ballot_summary1 = ballot_summary1.stdout
+
+    # ballots of new data
+    ballot_summary2 = subprocess.run(["belenios-tool", "election", "compute-ballot-summary", "--dir={}".format(pnew)],
+                                     stdout=subprocess.PIPE, stderr=subprocess.DEVNULL)
+    if ballot_summary2.returncode != 0:
+        msg = "Error: compute-ballot-summary on new data failed for election {}".format(uuid).encode()
+        return Status(True, msg)
+    ballot_summary2 = ballot_summary2.stdout
+
+    # compute new ballots
+    new_ballots = get_new_ballots(ballot_summary1, ballot_summary2)
     data['new_ballots'] = new_ballots
+    data['ballot_summary'] = ballot_summary2
+
+    # compute checksums
+    checksums = subprocess.run(["belenios-tool", "election", "compute-checksums", "--dir={}".format(pnew)],
+                               stdout=subprocess.PIPE, stderr=subprocess.DEVNULL)
+    if checksums.returncode != 0:
+        msg = "Error: belenios-tool election compute-checksums failed on newly downloaded data form election {}, with output {}".format(uuid, checksums.stdout).encode()
+        return Status(True, msg)
+    data["checksums"] = checksums.stdout
 
     # move new files to main subdirectory
     for f in audit_files + optional_audit_files:
@@ -196,20 +220,11 @@ def check_hash_ballots(data):
 
     list_hash2 = []
     list_weights2 = []
-    for l in data['ballots.jsons'].splitlines():
-        m = hashlib.sha256()
-        m.update(l)
-        h = base64.b64encode(m.digest()).decode().strip('=')
+    for x in json.loads(data['ballot_summary']):
+        h = b64_of_hex(x["hash"])
         list_hash2.append(h)
         if has_weight:
-            jsn = json.loads(l)
-            cred = jsn['credential']
-            pat = re.compile(cred + r',(\d+)')
-            m=pat.search(data['public_creds.txt'].decode())
-            if m:
-                list_weights2.append(m.group(1))
-            else: # if absent, default weight is 1
-                list_weights2.append('1')
+            list_weights2.append(str(x["weight"]))
     if has_weight:
         list_hash = [ (list_hash[i],list_weights[i]) for i in range(len(list_hash)) ]
         list_hash2 = [ (list_hash2[i],list_weights2[i]) for i in range(len(list_hash2)) ]
@@ -236,10 +251,11 @@ def check_index_html(data):
 
     logme("Checking index.html of {} ...".format(uuid))
 
+    # load checksums computed by belenios-tool
+    checksums = json.loads(data["checksums"])
+
     # fingerprint of the election vs election.json
-    m = hashlib.sha256()
-    m.update(data['election.json'][0:-1]) # remove trailing \n
-    h = base64.b64encode(m.digest()).decode().strip('=')
+    h = b64_of_hex(checksums["election"])
     h2 = dom.getElementsByTagName("code")[0].firstChild.data
     if (not h == h2):
         msg = "Error: Wrong fingerprint of election {}\n".format(uuid).encode()
@@ -247,14 +263,9 @@ def check_index_html(data):
     else:
         logme("  election fingerprint ok")
 
-    # credential fingerprint vs public_creds.txt
-    m = hashlib.sha256()
-    m.update(data['public_creds.txt'])
-    h = base64.b64encode(m.digest()).decode().strip('=')
-    node = [ x.firstChild for x in dom.getElementsByTagName("div")
-            if x.firstChild != None and
-            x.firstChild.nodeType == xml.dom.minidom.Node.TEXT_NODE and
-            re.search("Credentials were generated",x.firstChild.data) != None ]
+    # credential fingerprint
+    h = b64_of_hex(checksums["public_credentials"])
+    node = [ x.firstChild for x in dom.getElementsByTagName("div") if x.getAttribute("id") == "credentials" ]
     assert len(node) == 1
     h2 = node[0].data.split(' ')[-1].strip('.')
     if (not h == h2):
@@ -263,52 +274,38 @@ def check_index_html(data):
     else:
         logme("  cred fingerprint ok")
 
-    # if weights, check the total/min/max vs content of public_creds.txt
-    node = [ x.firstChild for x in dom.getElementsByTagName("div")
-            if x.firstChild != None and
-            x.firstChild.nodeType == xml.dom.minidom.Node.TEXT_NODE and
-            re.search("The total weight is",x.firstChild.data) != None ]
-    if (len(node) == 1):
-        fail_weights = False
-        pat = re.compile(r'The total weight is (\d+) \(min: (\d+), max: (\d+)\)')
-        mat = pat.match(node[0].data)
-        w_tot = int(mat.group(1))
-        w_min = int(mat.group(2))
-        w_max = int(mat.group(3))
-        c_tot = 0
-        c_min = 10000000000
-        c_max = 0
-        for l in data['public_creds.txt'].splitlines():
-            wt = l.decode().split(',')
-            if len(wt) == 2:
-                x = int(l.decode().split(',')[1])
-            else: # if absent, default weight is 1.
-                assert len(wt) == 1;
-                x = 1
-            c_tot += x
-            if (x < c_min):
-                c_min = x
-            if (x > c_max):
-                c_max = x
-        if (c_tot != w_tot) or (c_min != w_min) or (c_max != w_max):
-            msg = msg + "Error: Wrong stats of weights in election {}\n".format(uuid).encode()
-            logme(" " + str(c_tot) + " " + str(c_min) + " " + str(c_max))
-            fail = True
+    # if weights, check the total/min/max
+    if "weights" in checksums:
+        weights = checksums["weights"]
+        node = [ x.firstChild for x in dom.getElementsByTagName("div") if x.getAttribute("id") == "weights" ]
+        if len(node) == 1:
+            pat = re.compile(r'The total weight is (\d+) \(min: (\d+), max: (\d+)\)')
+            mat = pat.match(node[0].data)
+            w_tot = int(mat.group(1))
+            w_min = int(mat.group(2))
+            w_max = int(mat.group(3))
+            c_tot = int(weights["total"])
+            c_min = int(weights["min"])
+            c_max = int(weights["max"])
+            if (c_tot != w_tot) or (c_min != w_min) or (c_max != w_max):
+                msg = msg + "Error: Wrong stats of weights in election {}\n".format(uuid).encode()
+                logme(" " + str(c_tot) + " " + str(c_min) + " " + str(c_max))
+                fail = True
+            else:
+                logme("  stats of weights ok")
         else:
-            logme("  stats of weights ok")
+            msg += "Error: missing weights in election {}\n".format(uuid).encode()
+            fail = True
 
     # check that the printed number of voters is consistent with number
     # of credentials
-    node = [ x.firstChild for x in dom.getElementsByTagName("div")
-            if x.firstChild != None and
-            x.firstChild.nodeType == xml.dom.minidom.Node.TEXT_NODE and
-            re.search("The voter list has",x.firstChild.data) != None ]
+    node = [ x.firstChild for x in dom.getElementsByTagName("div") if x.getAttribute("id") == "voters" ]
     assert len(node) == 1
     pat = re.compile(r'The voter list has (\d+) voter\(s\) and fingerprint ([\w/+]+).')
     mat = pat.match(node[0].data)
     nb_voters = int(mat.group(1))
     data['hash_voterlist'] = mat.group(2).encode() # for further checks
-    nb_creds = data['public_creds.txt'].decode().count('\n')
+    nb_creds = checksums["num_voters"]
     if (nb_voters != nb_creds):
         msg = msg + "Error: Number of voters different from number of credentials for election {}\n".format(uuid).encode()
         logme(" " + str(nb_voters) + " " + str(nb_creds))
@@ -316,39 +313,27 @@ def check_index_html(data):
     else:
         logme("  number of voters ok")
 
-    def hash_pub_key(s):
-        m = hashlib.sha256()
-        m.update(b'\"')
-        m.update(s.encode())
-        m.update(b'\"')
-        return base64.b64encode(m.digest()).decode().strip('=')
-
     # trustees fingerprint vs trustees.json
-    jsn = json.loads(data['trustees.json'])
     names = []
     pks = []
     certs = []
 
-    for trustee in jsn:
-        if trustee[0] == 'Single':
-            if 'name' in trustee[1]:
-                names.append(trustee[1]['name'])
+    for trustee in checksums["trustees"]:
+        if "name" in trustee:
+            names.append(trustee["name"])
+        else:
+            names.append("N/A")
+        pks.append(b64_of_hex(trustee["checksum"]))
+        certs.append(None)
+
+    for tset in checksums["trustees_threshold"]:
+        for trustee in tset["trustees"]:
+            if "name" in trustee:
+                names.append(trustee["name"])
             else:
                 names.append("N/A")
-            pks.append(hash_pub_key(trustee[1]['public_key']))
-            certs.append(None)
-        else:
-            assert trustee[0] == 'Pedersen'
-            for tru in trustee[1]['verification_keys']:
-                if 'name' in tru:
-                    names.append(tru['name'])
-                else:
-                    names.append("N/A")
-                pks.append(hash_pub_key(tru['public_key']))
-            for tru in trustee[1]['certs']:
-                m = hashlib.sha256()
-                m.update(tru['message'].encode())
-                certs.append(base64.b64encode(m.digest()).decode().strip('='))
+            pks.append(b64_of_hex(trustee["verification_key"]))
+            certs.append(b64_of_hex(trustee["pki_key"]))
 
     names2 = []
     pks2 = []
@@ -380,48 +365,23 @@ def check_index_html(data):
     else:
         logme("  trustees fingerprints ok")
 
-    # encrypted tally vs result.json (if present)
-    # (and extract shuffles if present)
-    shuf_array = None
-    if data['result.json'] != b'':
-        jsn = json.loads(data['result.json'])
-        if 'encrypted_tally' in jsn:
-            s = jsn['encrypted_tally']
-            # convert to a string as Belenios use it for hashing
-            ss = json.JSONEncoder().encode(s).replace(' ', '')
-            m = hashlib.sha256()
-            m.update(ss.encode())
-            h = base64.b64encode(m.digest()).decode().strip('=')
-            node = [ x.firstChild for x in dom.getElementsByTagName("div") if
-                    x.firstChild != None and x.firstChild.nodeType ==
-                    xml.dom.minidom.Node.TEXT_NODE and
-                    re.search("The fingerprint of the encrypted tally",
-                        x.firstChild.data) != None ]
-            assert len(node) == 1
+    # check consistency of encrypted tally
+    if "encrypted_tally" in checksums:
+        h = b64_of_hex(checksums['encrypted_tally'])
+        node = [ x.firstChild for x in dom.getElementsByTagName("div") if x.getAttribute("id") == "encrypted_tally" ]
+        assert len(node) <= 1
+        if len(node) == 1:
             h2 = node[0].data.split(' ')[-1].strip('.')
             if (not h == h2):
                 msg = msg + "Error: Wrong encrypted tally fingerprint of election {}\n".format(uuid).encode()
                 fail = True
             else:
                 logme("  encrypted tally fingerprint ok")
-        if 'shuffles' in jsn:
-            shuf_array=jsn['shuffles']
-    elif data['shuffles.jsons'] != b'':
-        # not exactly the same as in result.json: one json per line...
-        shuf_array = []
-        for line in data['shuffles.jsons'].decode().splitlines():
-            shuf_array.append(json.loads(line))
 
-    # shuffles fingerprint vs result.json or shuffles.jsons (if present)
-    if shuf_array != None:
-        hashs = []
+    # check consistency of shuffles
+    if "shuffles" in checksums:
+        hashs = [b64_of_hex(x["checksum"]) for x in checksums["shuffles"]]
         hashs2 = []
-        for s in shuf_array:
-            # convert to a string as Belenios use it for hashing
-            ss = json.JSONEncoder().encode(s).replace(' ', '')
-            m = hashlib.sha256()
-            m.update(ss.encode())
-            hashs.append(base64.b64encode(m.digest()).decode().strip('='))
         # in index.html, the shuffles are in the ul with id 'shuffles'
         for shuf in [ x for x in dom.getElementsByTagName("ul") if
                 x.getAttribute('id') == 'shuffles'
@@ -492,7 +452,7 @@ def hash_votefile(link, lang):
         resp = urllib.request.urlopen(req)
         data = resp.read()
     except:
-        print("Failed to download vote.html in lang {}.  Aborting".format(lang))
+        print("Failed to download {} in lang {}.  Aborting".format(link, lang))
         sys.exit(1)
     m = hashlib.sha256()
     m.update(data)
@@ -634,7 +594,7 @@ if args.checkhash == True:
         else:
             reference[f] = descr
     new_reference = {}
-    for f, descr in reference.items():
+    for f, descr in shuffle(reference.items()):
         if type(descr) == dict:
             new_reference[f] = {}
             if len(descr) == 0:
@@ -642,7 +602,7 @@ if args.checkhash == True:
                     langs = get_all_available_languages(args.beleniospath)
                 for lang in langs:
                     descr[lang] = None
-            for lang, descr in descr.items():
+            for lang, descr in shuffle(descr.items()):
                 h = hash_votefile(url + f, lang)
                 new_reference[f][lang] = h
                 if h != descr:
@@ -663,7 +623,7 @@ if args.checkhash == True:
     if hashfile_changed and args.outputref:
         print("Writing new reference file")
         with open(args.outputref, mode="w") as f:
-            json.dump(new_reference, f)
+            json.dump(new_reference, f, sort_keys=True)
 
     # If we can check signature, do it
     if args.sighashref:
@@ -717,10 +677,10 @@ for uuid in uuids:
         else:
             with open(p, "wb") as file:
                 file.write(data['hash_voterlist'])
-        
+
         # create the all_ballot_hashs file, or update it from the new
         # ballot files. Check that an old ballot was not replayed.
-        # Note: the list of new ballot hashs is created earlier, during 
+        # Note: the list of new ballot hashs is created earlier, during
         # write_and_verify_new_data(), because it must compare the old
         # and new ballot box.
         p = os.path.join(args.wdir, uuid, 'all_ballot_hashs')
