@@ -19,25 +19,17 @@
 (*  <http://www.gnu.org/licenses/>.                                       *)
 (**************************************************************************)
 
-open Belenios_core
 open Belenios
-open Signatures
+open Belenios_core.Signatures
 open Belenios_platform
 open Belenios_tool_common
-open Serializable_j
+open Belenios_core.Serializable_builtin_t
+open Belenios_core.Serializable_j
 open Platform
+open Belenios_core.Common
 open Common
 open Cmdliner
 
-let lines_of_file fname =
-  let ic = open_in fname in
-  let rec loop accu =
-    match input_line ic with
-    | line -> loop (line :: accu)
-    | exception End_of_file -> close_in ic; List.rev accu
-  in
-  loop []
-
 let lines_of_stdin () =
   let rec loop accu =
     match input_line stdin with
@@ -56,21 +48,10 @@ let chars_of_stdin () =
   loop ();
   Buffer.contents buf
 
-let string_of_file f =
-  lines_of_file f |> String.concat "\n"
-
-let load_from_file of_string filename =
-  if Sys.file_exists filename then (
-    Printf.eprintf "I: loading %s...\n%!" (Filename.basename filename);
-    Some (lines_of_file filename |> List.rev_map of_string)
-  ) else None
-
-let ( / ) = Filename.concat
-
 let download dir url file =
   let url = if url.[String.length url - 1] = '/' then url else url ^ "/" in
   Printf.eprintf "I: downloading %s...\n%!" file;
-  let target = dir / file in
+  let target = dir // file in
   let command =
     Printf.sprintf "curl --silent --fail \"%s%s\" > \"%s\"" url file target
   in
@@ -79,36 +60,13 @@ let download dir url file =
 
 let rm_rf dir =
   let files = Sys.readdir dir in
-  Array.iter (fun f -> Unix.unlink (dir / f)) files;
+  Array.iter (fun f -> Unix.unlink (dir // f)) files;
   Unix.rmdir dir
 
-exception Cmdline_error of string
-
-let failcmd fmt = Printf.ksprintf (fun x -> raise (Cmdline_error x)) fmt
-
-let common_man = [
-  `S "MORE INFORMATION";
-  `P "This command is part of the Belenios command-line tool.";
-  `P "To get more help on a specific subcommand, run:";
-  `P "$(b,belenios-tool) $(i,COMMAND) $(b,--help)";
-  `P "See $(i,https://www.belenios.org/).";
-]
-
 let get_mandatory_opt name = function
   | Some x -> x
   | None -> failcmd "%s is mandatory" name
 
-let wrap_main f =
-  match f () with
-  | () -> `Ok ()
-  | exception Cmdline_error e -> `Error (true, e)
-  | exception Failure e -> `Error (false, e)
-  | exception e -> `Error (false, Printexc.to_string e)
-
-module type CMDLINER_MODULE = sig
-  val cmds : (unit Cmdliner.Term.t * Cmdliner.Cmd.info) list
-end
-
 module Shasum : CMDLINER_MODULE = struct
 
   let main () =
@@ -123,16 +81,16 @@ module Shasum : CMDLINER_MODULE = struct
         `P "but does not need each individual command to be available.";
       ] @ common_man
     in
-    Term.(ret (const main $ const ())),
-    Cmd.info "sha256-b64" ~doc ~man
+    Cmd.v (Cmd.info "sha256-b64" ~doc ~man)
+      Term.(ret (const main $ const ()))
 
   let cmds = [sha256_b64_cmd]
 
 end
 
 let group_t =
-  let doc = "Take group parameters from file $(docv)." in
-  Arg.(value & opt (some file) None & info ["group"] ~docv:"GROUP" ~doc)
+  let doc = "Use group $(docv)." in
+  Arg.(value & opt (some string) None & info ["group"] ~docv:"GROUP" ~doc)
 
 let version_t =
   let doc = "Use protocol version $(docv)." in
@@ -164,7 +122,7 @@ module Tkeygen : CMDLINER_MODULE = struct
   let main group version =
     let@ () = wrap_main in
     let module P = struct
-        let group = get_mandatory_opt "--group" group |> string_of_file
+        let group = get_mandatory_opt "--group" group
         let version = version
       end in
     let module R = Make (P) (Random) () in
@@ -190,8 +148,8 @@ module Tkeygen : CMDLINER_MODULE = struct
       `S "DESCRIPTION";
       `P "This command is run by a trustee to generate a share of an election key. Such a share consists of a private key and a public key with a certificate. Generated files are stored in the current directory with a name that starts with $(i,ID), where $(i,ID) is a short fingerprint of the public key. The private key is stored in $(i,ID.privkey) and must be secured by the trustee. The public key is stored in $(i,ID.pubkey) and must be sent to the election administrator.";
     ] @ common_man in
-    Term.(ret (const main $ group_t $ version_t)),
-    Cmd.info "trustee-keygen" ~doc ~man
+    Cmd.v (Cmd.info "generate-trustee-key" ~doc ~man)
+      Term.(ret (const main $ group_t $ version_t))
 
   let cmds = [tkeygen_cmd]
 
@@ -213,7 +171,7 @@ module Ttkeygen : CMDLINER_MODULE = struct
       | None -> Printf.ksprintf failwith "%s does not exist" polynomials
       | Some l -> Array.of_list (List.rev l)
     in
-    let group = get_mandatory_opt "--group" group |> string_of_file in
+    let group = get_mandatory_opt "--group" group in
     let module G = (val Group.of_string ~version group : GROUP) in
     let module Trustees = (val Trustees.get_by_version version) in
     let module P = Trustees.MakePKI (G) (Random) in
@@ -318,51 +276,16 @@ module Ttkeygen : CMDLINER_MODULE = struct
         `S "DESCRIPTION";
         `P "This command is run by trustees and the administrator to generate an election key with threshold decryption.";
       ] @ common_man in
-    Term.(ret (const main $ group_t $ version_t $ step_t $ cert_t $ threshold_t $ key_t $ polynomials_t)),
-    Cmd.info "threshold-trustee-keygen" ~doc ~man
+    Cmd.v (Cmd.info "generate-trustee-key-threshold" ~doc ~man)
+      Term.(ret (const main $ group_t $ version_t $ step_t $ cert_t $ threshold_t $ key_t $ polynomials_t))
 
   let cmds = [ttkeygen_cmd]
 
 end
 
-module Election : CMDLINER_MODULE = struct
+module ElectionManagement : CMDLINER_MODULE = struct
   open Tool_election
 
-  module MakeGetters (X : sig val dir : string end) = struct
-
-    let get_public_creds () =
-      let file = "public_creds.txt" in
-      Printf.eprintf "I: loading %s...\n%!" file;
-      try Some (lines_of_file (X.dir / file)) with _ -> None
-
-    let get_trustees () =
-      let file = "trustees.json" in
-      Printf.eprintf "I: loading %s...\n%!" file;
-      try Some (string_of_file (X.dir / file)) with _ -> None
-
-    let get_ballots () =
-      let file = "ballots.jsons" in
-      Printf.eprintf "I: loading %s...\n%!" file;
-      try Some (lines_of_file (X.dir / file)) with _ -> None
-
-    let get_shuffles () =
-      let file = "shuffles.jsons" in
-      if Sys.file_exists (X.dir / file) then (
-        Printf.eprintf "I: loading %s...\n%!" file;
-        try Some (lines_of_file (X.dir / file))
-        with _ -> None
-      ) else None
-
-    let get_result () =
-      load_from_file (fun x -> x) (X.dir/"result.json") |> function
-      | None -> None
-      | Some [r] -> Some r
-      | _ -> failwith "invalid result"
-
-    let print_msg = prerr_endline
-
-  end
-
   let main url dir action =
     let@ () = wrap_main in
     let dir, cleanup = match url, dir with
@@ -388,17 +311,7 @@ module Election : CMDLINER_MODULE = struct
               ) then
            Printf.eprintf "W: some errors occurred while downloading\n%!";
     in
-    let module P : PARAMS = struct
-        include MakeGetters (struct let dir = dir end)
-        let raw_election =
-          let fname = dir/"election.json" in
-          load_from_file (fun x -> x) fname |>
-            function
-            | Some [e] -> e
-            | None -> failcmd "could not read %s" fname
-            | _ -> Printf.ksprintf failwith "invalid election file: %s" fname
-      end in
-    let module X = Make (P) (Random) () in
+    let module X = Make (struct let dir = dir end) () in
     begin match action with
     | `Vote (privcred, ballot) ->
        let ballot =
@@ -426,13 +339,13 @@ module Election : CMDLINER_MODULE = struct
     | `Verify -> X.verify ()
     | `Validate ->
        let factors =
-         let fname = dir/"partial_decryptions.jsons" in
+         let fname = dir // "partial_decryptions.jsons" in
          match load_from_file (fun x -> x) fname with
          | Some factors -> factors
          | None -> failwith "cannot load partial decryptions"
        in
        let result = X.validate factors in
-       let oc = open_out (dir/"result.json") in
+       let oc = open_out (dir // "result.json") in
        output_string oc result;
        output_char oc '\n';
        close_out oc
@@ -443,6 +356,8 @@ module Election : CMDLINER_MODULE = struct
        X.checksums () |> print_endline
     | `ComputeVoters privcreds ->
        X.compute_voters privcreds |> List.iter print_endline
+    | `ComputeBallotSummary ->
+       X.compute_ballot_summary () |> print_endline
     end;
     if cleanup then rm_rf dir
 
@@ -482,8 +397,8 @@ module Election : CMDLINER_MODULE = struct
       let b = get_mandatory_opt "--ballot" b in
       main u d (`Vote (p, b))
     ) in
-    Term.(ret (main $ url_t $ optdir_t $ privcred_t $ ballot_t)),
-    Cmd.info "vote" ~doc ~man
+    Cmd.v (Cmd.info "generate-ballot" ~doc ~man)
+      Term.(ret (main $ url_t $ optdir_t $ privcred_t $ ballot_t))
 
   let verify_cmd =
     let doc = "verify election data" in
@@ -491,8 +406,8 @@ module Election : CMDLINER_MODULE = struct
       `S "DESCRIPTION";
       `P "This command performs all possible verifications.";
     ] @ common_man in
-    Term.(ret (const main $ url_t $ optdir_t $ const `Verify)),
-    Cmd.info "verify" ~doc ~man
+    Cmd.v (Cmd.info "verify" ~doc ~man)
+      Term.(ret (const main $ url_t $ optdir_t $ const `Verify))
 
   let decrypt_man = [
       `S "DESCRIPTION";
@@ -505,8 +420,8 @@ module Election : CMDLINER_MODULE = struct
       let p = get_mandatory_opt "--privkey" p in
       main u d (`Decrypt p)
     ) in
-    Term.(ret (main $ url_t $ optdir_t $ privkey_t)),
-    Cmd.info "decrypt" ~doc ~man:decrypt_man
+    Cmd.v (Cmd.info "decrypt" ~doc ~man:decrypt_man)
+      Term.(ret (main $ url_t $ optdir_t $ privkey_t))
 
   let tdecrypt_cmd =
     let doc = "perform partial decryption (threshold version)" in
@@ -516,8 +431,8 @@ module Election : CMDLINER_MODULE = struct
                    main u d (`TDecrypt (k, pdk))
                  )
     in
-    Term.(ret (main $ url_t $ optdir_t $ key_t $ pdk_t)),
-    Cmd.info "threshold-decrypt" ~doc ~man:decrypt_man
+    Cmd.v (Cmd.info "decrypt-threshold" ~doc ~man:decrypt_man)
+      Term.(ret (main $ url_t $ optdir_t $ key_t $ pdk_t))
 
   let validate_cmd =
     let doc = "validates an election" in
@@ -526,8 +441,8 @@ module Election : CMDLINER_MODULE = struct
       `P "This command reads partial decryptions done by trustees from file $(i,partial_decryptions.jsons), checks them, combines them into the final tally and prints the result to standard output.";
       `P "The result structure contains partial decryptions itself, so $(i,partial_decryptions.jsons) can be discarded afterwards.";
     ] @ common_man in
-    Term.(ret (const main $ url_t $ optdir_t $ const `Validate)),
-    Cmd.info "validate" ~doc ~man
+    Cmd.v (Cmd.info "compute-result" ~doc ~man)
+      Term.(ret (const main $ url_t $ optdir_t $ const `Validate))
 
   let shuffle_cmd =
     let doc = "shuffle ciphertexts" in
@@ -536,8 +451,8 @@ module Election : CMDLINER_MODULE = struct
         `P "This command shuffles non-homomorphic ciphertexts and prints on standard output the shuffle proof and the shuffled ciphertexts.";
       ] @ common_man
     in
-    Term.(ret (const main $ url_t $ optdir_t $ const `Shuffle)),
-    Cmd.info "shuffle" ~doc ~man
+    Cmd.v (Cmd.info "shuffle" ~doc ~man)
+      Term.(ret (const main $ url_t $ optdir_t $ const `Shuffle))
 
   let checksums_cmd =
     let doc = "compute checksums" in
@@ -546,8 +461,8 @@ module Election : CMDLINER_MODULE = struct
         `P "This command computes checksums needed to audit an election.";
       ] @ common_man
     in
-    Term.(ret (const main $ url_t $ optdir_t $ const `Checksums)),
-    Cmd.info "checksums" ~doc ~man
+    Cmd.v (Cmd.info "compute-checksums" ~doc ~man)
+      Term.(ret (const main $ url_t $ optdir_t $ const `Checksums))
 
   let compute_voters_cmd =
     let doc = "compute actual voters" in
@@ -566,8 +481,18 @@ module Election : CMDLINER_MODULE = struct
           main u d (`ComputeVoters privcreds)
         )
     in
-    Term.(ret (main $ url_t $ optdir_t $ privcreds_t)),
-    Cmd.info "compute-voters" ~doc ~man
+    Cmd.v (Cmd.info "compute-voters" ~doc ~man)
+      Term.(ret (main $ url_t $ optdir_t $ privcreds_t))
+
+  let compute_ballot_summary_cmd =
+    let doc = "compute ballot summary" in
+    let man = [
+        `S "DESCRIPTION";
+        `P "This command compute the hash (also known as smart ballot tracker) and weight of all ballots.";
+      ] @ common_man
+    in
+    Cmd.v (Cmd.info "compute-ballot-summary" ~doc ~man)
+      Term.(ret (const main $ url_t $ optdir_t $ const `ComputeBallotSummary))
 
   let cmds =
     [
@@ -579,6 +504,7 @@ module Election : CMDLINER_MODULE = struct
       shuffle_cmd;
       checksums_cmd;
       compute_voters_cmd;
+      compute_ballot_summary_cmd;
     ]
 
 end
@@ -605,7 +531,7 @@ module Credgen : CMDLINER_MODULE = struct
     let@ () = wrap_main in
     let module P = struct
         let version = version
-        let group = get_mandatory_opt "--group" group |> string_of_file
+        let group = get_mandatory_opt "--group" group
         let uuid = get_mandatory_opt "--uuid" uuid
       end in
     let module R = Make (P) (Random) () in
@@ -630,7 +556,7 @@ module Credgen : CMDLINER_MODULE = struct
          |> List.map (fun (id, priv) -> id ^ " " ^ priv)
        in
        let timestamp = Printf.sprintf "%.0f" (Unix.time ()) in
-       let base = dir / timestamp in
+       let base = dir // timestamp in
        save params_priv base privs;
        save params_pub base pubs
 
@@ -655,8 +581,8 @@ module Credgen : CMDLINER_MODULE = struct
       `S "DESCRIPTION";
       `P "This command is run by a credential authority to generate credentials for a specific election. The generated private credentials are stored in $(i,T.privcreds), where $(i,T) is a timestamp. $(i,T.privcreds) contains one credential per line. Each voter must be sent a credential, and $(i,T.privcreds) must be destroyed after dispatching is done. The associated public keys are stored in $(i,T.pubcreds) and must be sent to the election administrator.";
     ] @ common_man in
-    Term.(ret (const main $ version_t $ group_t $ dir_t $ uuid_t $ count_t $ file_t $ derive_t)),
-    Cmd.info "credgen" ~doc ~man
+    Cmd.v (Cmd.info "generate-credentials" ~doc ~man)
+      Term.(ret (const main $ version_t $ group_t $ dir_t $ uuid_t $ count_t $ file_t $ derive_t))
 
   let cmds = [credgen_cmd]
 
@@ -666,10 +592,10 @@ module Mktrustees : CMDLINER_MODULE = struct
   let main dir =
     let@ () = wrap_main in
     let get_public_keys () =
-      Some (lines_of_file (dir / "public_keys.jsons"))
+      Some (lines_of_file (dir // "public_keys.jsons"))
     in
     let get_threshold () =
-      let fn = dir / "threshold.json" in
+      let fn = dir // "threshold.json" in
       if Sys.file_exists fn then Some (string_of_file fn) else None
     in
     let get_trustees () =
@@ -694,7 +620,7 @@ module Mktrustees : CMDLINER_MODULE = struct
       | trustees -> string_of_trustees Yojson.Safe.write_json trustees
     in
     let trustees = get_trustees () in
-    let oc = open_out (dir / "trustees.json") in
+    let oc = open_out (dir // "trustees.json") in
     output_string oc trustees;
     output_char oc '\n';
     close_out oc
@@ -705,8 +631,8 @@ module Mktrustees : CMDLINER_MODULE = struct
       `S "DESCRIPTION";
       `P "This command reads $(i,public_keys.jsons) and $(i,threshold.json) (if any). It then generates an $(i,trustees.json) file.";
     ] @ common_man in
-    Term.(ret (const main $ dir_t)),
-    Cmd.info "mktrustees" ~doc ~man
+    Cmd.v (Cmd.info "make-trustees" ~doc ~man)
+      Term.(ret (const main $ dir_t))
 
   let cmds = [mktrustees_cmd]
 
@@ -719,11 +645,11 @@ module Mkelection : CMDLINER_MODULE = struct
     let@ () = wrap_main in
     let module P = struct
         let version = version
-        let group = get_mandatory_opt "--group" group |> string_of_file
+        let group = get_mandatory_opt "--group" group
         let uuid = get_mandatory_opt "--uuid" uuid
         let template = get_mandatory_opt "--template" template |> string_of_file
         let get_trustees () =
-          let fn = dir / "trustees.json" in
+          let fn = dir // "trustees.json" in
           if Sys.file_exists fn then
             string_of_file fn
           else
@@ -731,7 +657,7 @@ module Mkelection : CMDLINER_MODULE = struct
       end in
     let module R = (val make (module P : PARAMS) : S) in
     let params = R.mkelection () in
-    let oc = open_out (dir / "election.json") in
+    let oc = open_out (dir // "election.json") in
     output_string oc params;
     output_char oc '\n';
     close_out oc
@@ -746,8 +672,8 @@ module Mkelection : CMDLINER_MODULE = struct
       `S "DESCRIPTION";
       `P "This command reads and checks $(i,public_keys.jsons) (or $(i,threshold.json) if it exists). It then computes the global election public key and generates an $(i,election.json) file.";
     ] @ common_man in
-    Term.(ret (const main $ dir_t $ group_t $ version_t $ uuid_t $ template_t)),
-    Cmd.info "mkelection" ~doc ~man
+    Cmd.v (Cmd.info "make-election" ~doc ~man)
+      Term.(ret (const main $ dir_t $ group_t $ version_t $ uuid_t $ template_t))
 
   let cmds = [mkelection_cmd]
 
@@ -776,8 +702,8 @@ module Verifydiff : CMDLINER_MODULE = struct
         `S "DESCRIPTION";
         `P "This command is run by an auditor on two directories $(i,DIR1) and $(i,DIR2). It checks that $(i,DIR2) is a valid update of $(i,DIR1).";
       ] @ common_man in
-    Term.(ret (const main $ dir1_t $ dir2_t)),
-    Cmd.info "verify-diff" ~doc ~man
+    Cmd.v (Cmd.info "verify-diff" ~doc ~man)
+      Term.(ret (const main $ dir1_t $ dir2_t))
 
   let cmds = [verifydiff_cmd]
 
@@ -802,7 +728,7 @@ module Methods : CMDLINER_MODULE = struct
         | Some b -> b
       in
       ballots
-      |> Schulze.compute ~nchoices ~blank_allowed
+      |> Belenios_core.Schulze.compute ~nchoices ~blank_allowed
       |> string_of_schulze_result
       |> print_endline
 
@@ -828,7 +754,7 @@ module Methods : CMDLINER_MODULE = struct
         | Some b -> b
       in
       ballots
-      |> Majority_judgment.compute ~nchoices ~ngrades ~blank_allowed
+      |> Belenios_core.Majority_judgment.compute ~nchoices ~ngrades ~blank_allowed
       |> string_of_mj_result
       |> print_endline
 
@@ -841,7 +767,7 @@ module Methods : CMDLINER_MODULE = struct
     in
     chars_of_stdin ()
     |> stv_raw_ballots_of_string
-    |> Stv.compute ~nseats
+    |> Belenios_core.Stv.compute ~nseats
     |> string_of_stv_result
     |> print_endline
 
@@ -868,8 +794,8 @@ module Methods : CMDLINER_MODULE = struct
         `P "This command reads on standard input JSON-formatted ballots and interprets them as Condorcet rankings on $(i,N) choices. It then computes the result according to the Schulze method and prints it on standard output.";
       ] @ common_man
     in
-    Term.(ret (const schulze $ nchoices_t $ blank_allowed_t)),
-    Cmd.info "method-schulze" ~doc ~man
+    Cmd.v (Cmd.info "schulze" ~doc ~man)
+      Term.(ret (const schulze $ nchoices_t $ blank_allowed_t))
 
   let mj_cmd =
     let doc = "compute Majority Judgment result" in
@@ -878,8 +804,8 @@ module Methods : CMDLINER_MODULE = struct
         `P "This command reads on standard input JSON-formatted ballots and interprets them as grades (ranging from 1 (best) to $(i,G) (worst)) given to $(i,N) choices. It then computes the result according to the Majority Judgment method and prints it on standard output.";
       ] @ common_man
     in
-    Term.(ret (const mj $ nchoices_t $ ngrades_t $ blank_allowed_t)),
-    Cmd.info "method-majority-judgment" ~doc ~man
+    Cmd.v (Cmd.info "majority-judgment" ~doc ~man)
+      Term.(ret (const mj $ nchoices_t $ ngrades_t $ blank_allowed_t))
 
   let stv_cmd =
     let doc = "compute Single Transferable Vote result" in
@@ -888,10 +814,16 @@ module Methods : CMDLINER_MODULE = struct
         `P "This command reads on standard input JSON-formatted ballots and interprets them as rankings of choices (ranging from 1 (best) to $(i,X) (worst)). It then computes the result according to the Single Transferable Vote method and prints it on standard output.";
       ] @ common_man
     in
-    Term.(ret (const stv $ nseats_t)),
-    Cmd.info "method-stv" ~doc ~man
+    Cmd.v (Cmd.info "stv" ~doc ~man)
+      Term.(ret (const stv $ nseats_t))
 
-  let cmds = [schulze_cmd; mj_cmd; stv_cmd]
+  let method_cmd =
+    let doc = "compute result with specific counting methods" in
+    let man = common_man in
+    let info = Cmd.info "method" ~doc ~man in
+    Cmd.group info [schulze_cmd; mj_cmd; stv_cmd]
+
+  let cmds = [method_cmd]
 
 end
 
@@ -914,26 +846,58 @@ module GenerateToken : CMDLINER_MODULE = struct
         `P "This command generates a random token suitable for an election identifier.";
       ] @ common_man
     in
-    Term.(ret (const main $ length_t)),
-    Cmd.info "generate-token" ~doc ~man
+    Cmd.v (Cmd.info "generate-token" ~doc ~man)
+      Term.(ret (const main $ length_t))
 
   let cmds = [generate_token_cmd]
 
 end
 
+module Setup : CMDLINER_MODULE = struct
+  let cmd =
+    let doc = "election setup commands" in
+    let man = common_man in
+    let info = Cmd.info "setup" ~doc ~man in
+    let cmds =
+      List.flatten
+        [
+          Tkeygen.cmds;
+          Ttkeygen.cmds;
+          Credgen.cmds;
+          Mktrustees.cmds;
+          Mkelection.cmds;
+          GenerateToken.cmds;
+        ]
+    in
+    Cmd.group info cmds
+
+  let cmds = [cmd]
+end
+
+module Election : CMDLINER_MODULE = struct
+  let cmd =
+    let doc = "election management commands" in
+    let man = common_man in
+    let info = Cmd.info "election" ~doc ~man in
+    let cmds =
+      List.flatten
+        [
+          ElectionManagement.cmds;
+          Verifydiff.cmds;
+        ]
+    in
+    Cmd.group info cmds
+
+  let cmds = [cmd]
+end
+
 let cmds =
   List.flatten
     [
       Shasum.cmds;
-      Tkeygen.cmds;
-      Ttkeygen.cmds;
+      Setup.cmds;
       Election.cmds;
-      Credgen.cmds;
-      Mktrustees.cmds;
-      Mkelection.cmds;
-      Verifydiff.cmds;
       Methods.cmds;
-      GenerateToken.cmds;
     ]
 
 let default_cmd =
@@ -947,7 +911,7 @@ let default_cmd =
 
 let root_cmd =
   let default, i = default_cmd in
-  Cmd.(group ~default i (List.map (fun (t, i) -> v i t) cmds))
+  Cmd.(group ~default i cmds)
 
 let () =
   exit (Cmd.eval root_cmd)
