@@ -22,11 +22,9 @@
 open Lwt.Syntax
 open Js_of_ocaml
 open Js_of_ocaml_lwt
-open Belenios_platform
 open Belenios_core
 open Belenios
 open Belenios_tool_js_common
-open Platform
 open Serializable_builtin_t
 open Serializable_j
 open Signatures
@@ -380,6 +378,7 @@ let createStartButton params intro_div qs =
   b##.onclick := Dom_html.handler (fun _ ->
     (match prompt (s_ "Please enter your credential:") with
      | Some cred ->
+        let cred = String.trim cred in
         (match Credential.parse cred with
          | `Valid -> proceedWithCredential params intro_div qs cred
          | `Invalid ->
