@@ -40,7 +40,7 @@ let default_question_h =
     {
       q_question = "Question?";
       q_min = 1;
-      q_max = 2;
+      q_max = 1;
       q_blank = None;
       q_answers;
     }
@@ -67,19 +67,19 @@ let default_schulze_specification =
 (* Getting the OCaml structure out of the DOM *)
 
 let extractAnswer a =
-  let& x = Dom_html.CoerceTo.input a in
+  let&& x = Dom_html.CoerceTo.input a in
   return (Js.to_string (x##.value))
 
 let extractQuestion q =
-  let& x = Dom_html.CoerceTo.input q in
+  let&& x = Dom_html.CoerceTo.input q in
   let q_question = Js.to_string (x##.value) in
-  let& p1 = q##.parentNode in
-  let& p2 = p1##.parentNode in
-  let& p2 = Dom.CoerceTo.element p2 in
+  let&& p1 = q##.parentNode in
+  let&& p2 = p1##.parentNode in
+  let&& p2 = Dom.CoerceTo.element p2 in
   let p2 = Dom_html.element p2 in
   let numeric selector error_msg =
-    let& x = p2##querySelector (Js.string selector) in
-    let& x = Dom_html.CoerceTo.input x in
+    let&& x = p2##querySelector (Js.string selector) in
+    let&& x = Dom_html.CoerceTo.input x in
     let x = Js.to_string x##.value in
     try return (int_of_string x) with
     | _ -> failwith (error_msg ^ ": " ^ x ^ ".")
@@ -89,37 +89,39 @@ let extractQuestion q =
     Array.init
       (answers##.length)
       (fun i ->
-       let a = let& x = answers##item (i) in extractAnswer x in
+       let a = let&& x = answers##item (i) in extractAnswer x in
        Js.Opt.get a (fun () -> failwith "extractQuestion"))
   in
-  match Js.Opt.to_option (p2##querySelector (Js.string ".question_blank")) with
-  | Some q_blank ->
-     let& q_blank = Dom_html.CoerceTo.input q_blank in
-     let q_blank = if Js.to_bool q_blank##.checked then Some true else None in
-     let& q_min = numeric ".question_min" (s_ "Invalid minimum number of choices") in
-     let& q_max = numeric ".question_max" (s_ "Invalid maximum number of choices") in
-     if not (q_min <= q_max) then
-       failwith (s_ "Minimum number of choices must be less than or equal to maximum number of choices!");
-     if (q_max = 0) then
-       failwith (s_ "Maximum number of choices must be greater than 0!");
-     if (q_max > Array.length q_answers) then
-       failwith (s_ "The given maximum is greater than the number of choices!");
-     let open Question_h_t in
-     return (Question.Homomorphic {q_question; q_blank; q_min; q_max; q_answers})
-  | None ->
-     let& x = p2##querySelector (Js.string ".question_extra") in
-     let& x = Dom_html.CoerceTo.input x in
-     let x = Js.to_string x##.value in
-     let extra =
-       if x = "" then
-         None
-       else (
-         try Some (Yojson.Safe.from_string x)
-         with _ -> failwith (s_ "Invalid counting method specification!")
-       )
-     in
-     let open Question_nh_t in
-     return (Question.NonHomomorphic ({q_question; q_answers}, extra))
+  Js.Opt.case (p2##querySelector (Js.string ".question_blank"))
+    (fun () ->
+      let&& x = p2##querySelector (Js.string ".question_extra") in
+      let&& x = Dom_html.CoerceTo.input x in
+      let x = Js.to_string x##.value in
+      let extra =
+        if x = "" then
+          None
+        else (
+          try Some (Yojson.Safe.from_string x)
+          with _ -> failwith (s_ "Invalid counting method specification!")
+        )
+      in
+      let open Question_nh_t in
+      return (Question.NonHomomorphic ({q_question; q_answers}, extra))
+    )
+    (fun q_blank ->
+      let&& q_blank = Dom_html.CoerceTo.input q_blank in
+      let q_blank = if Js.to_bool q_blank##.checked then Some true else None in
+      let&& q_min = numeric ".question_min" (s_ "Invalid minimum number of choices") in
+      let&& q_max = numeric ".question_max" (s_ "Invalid maximum number of choices") in
+      if not (q_min <= q_max) then
+        failwith (s_ "Minimum number of choices must be less than or equal to maximum number of choices!");
+      if (q_max = 0) then
+        failwith (s_ "Maximum number of choices must be greater than 0!");
+      if (q_max > Array.length q_answers) then
+        failwith (s_ "The given maximum is greater than the number of choices!");
+      let open Question_h_t in
+      return (Question.Homomorphic {q_question; q_blank; q_min; q_max; q_answers})
+    )
 
 let extractTemplate () =
   let t_name = get_input "election_name" in
@@ -129,7 +131,7 @@ let extractTemplate () =
     Array.init
       (questions##.length)
       (fun i ->
-       let q = let& x = questions##item (i) in extractQuestion x in
+       let q = let&& x = questions##item (i) in extractQuestion x in
        Js.Opt.get q (fun () -> failwith "extractTemplate"))
   in
   let t_administrator = None in
@@ -151,7 +153,7 @@ let rec createAnswer a =
   let btn_text = document##createTextNode (Js.string (s_ "Remove")) in
   let btn = Dom_html.createButton document in
   let f _ =
-    let& x = container##.parentNode in
+    let&& x = container##.parentNode in
     Dom.removeChild x container;
     return ()
   in
@@ -163,7 +165,7 @@ let rec createAnswer a =
   let insert_btn = Dom_html.createButton document in
   let f _ =
     let x = createAnswer "" in
-    let& p = container##.parentNode in
+    let&& p = container##.parentNode in
     Dom.insertBefore p x (Js.some container);
     return ()
   in
@@ -213,7 +215,7 @@ let gensym =
   fun () -> incr counter; !counter
 
 let deleteQuestion q =
-  let& x = q##.parentNode in
+  let&& x = q##.parentNode in
   Dom.removeChild x q;
   return ()
 
@@ -241,7 +243,7 @@ let rec createQuestion booth_set q =
   let remove_text = document##createTextNode (Js.string (s_ "Remove")) in
   let remove_btn = Dom_html.createButton document in
   let f _ =
-    let& x = container##.parentNode in
+    let&& x = container##.parentNode in
     Dom.removeChild x container;
     return ()
   in
@@ -252,7 +254,7 @@ let rec createQuestion booth_set q =
   let insert_btn = Dom_html.createButton document in
   let f _ =
     let x = createQuestion booth_set (default_question ()) in
-    let& p = container##.parentNode in
+    let&& p = container##.parentNode in
     Dom.insertBefore p x (Js.some container);
     return ()
   in
@@ -365,7 +367,7 @@ let rec createQuestion booth_set q =
   let f =
     handler
       (fun _ ->
-        let& parent = container##.parentNode in
+        let&& parent = container##.parentNode in
         if Js.to_bool cb_type##.checked then
           Dom.replaceChild parent (createQuestion booth_set default_question_nh) container
         else
@@ -501,8 +503,8 @@ let createTemplate (booth_get, booth_set, booth_select) template =
       set_textarea "questions" (string_of_template template);
       let booth_version = booth_get () in
       set_input "booth_version" (string_of_int booth_version);
-      let& x = document##querySelector (Js.string "form") in
-      let& x = Dom_html.CoerceTo.form x in
+      let&& x = document##querySelector (Js.string "form") in
+      let&& x = Dom_html.CoerceTo.form x in
       let () = x##submit in
       return ()
     with Failure e ->
@@ -522,16 +524,16 @@ let handle_hybrid booth_set e _ =
   hybrid_mode := Js.to_bool e##.checked;
   let qs = document##querySelectorAll (Js.string ".question") in
   for i = 0 to qs##.length do
-    ignore (let& x = qs##item i in deleteQuestion x)
+    ignore (let&& x = qs##item i in deleteQuestion x)
   done;
-  let& qsdiv = document##getElementById (Js.string "election_questions") in
+  let&& qsdiv = document##getElementById (Js.string "election_questions") in
   Dom.appendChild qsdiv (createQuestion booth_set (default_question ()));
   return ()
 
 (* Entry point *)
 
 let fill_interactivity () =
-  let& e = document##getElementById (Js.string "interactivity") in
+  let&& e = document##getElementById (Js.string "interactivity") in
   let t = template_of_string (get_textarea "questions") in
   let booth_version = int_of_string (get_input "booth_version") in
   let has_nh =
@@ -545,10 +547,10 @@ let fill_interactivity () =
   let (_, booth_set, _) as booth_selector = createBoothSelector booth_version in
   let div = createTemplate booth_selector t in
   Dom.appendChild e div;
-  let& x = document##querySelector (Js.string "form") in
+  let&& x = document##querySelector (Js.string "form") in
   x##.style##.display := Js.string "none";
-  let& e = document##getElementById (Js.string "hybrid_mode") in
-  let& e = Dom_html.CoerceTo.input e in
+  let&& e = document##getElementById (Js.string "hybrid_mode") in
+  let&& e = Dom_html.CoerceTo.input e in
   e##.checked := Js.bool !hybrid_mode;
   e##.onchange := handler (handle_hybrid booth_set e);
   return ()
