@@ -29,8 +29,6 @@ open Belenios_api.Serializable_j
 open Web_common
 open Api_generic
 
-let ( / ) = Filename.concat
-
 let with_administrator token metadata f =
   let@ token = Option.unwrap unauthorized token in
   match lookup_token token, metadata.e_owner with
@@ -49,21 +47,19 @@ let find_trustee_id uuid token =
      Lwt.return (find 1 tokens)
 
 let find_trustee_private_key uuid trustee_id =
-  let* x = Web_persist.get_private_keys uuid in
-  match x with
-  | None -> Lwt.return_none
-  | Some keys ->
-     (* there is one Pedersen trustee *)
-     let* trustees = Web_persist.get_trustees uuid in
-     let open Belenios_core.Serializable_j in
-     let trustees = trustees_of_string Yojson.Safe.read_json trustees in
-     let rec loop i ts =
-       match ts with
-       | [] -> Lwt.return_none (* an error, actually *)
-       | `Single _ :: ts -> loop (i - 1) ts
-       | `Pedersen _ :: _ -> Lwt.return_some (List.nth keys i)
-     in
-     loop (trustee_id - 1) trustees
+  let* keys = Web_persist.get_private_keys uuid in
+  let&* keys in
+  (* there is one Pedersen trustee *)
+  let* trustees = Web_persist.get_trustees uuid in
+  let open Belenios_core.Serializable_j in
+  let trustees = trustees_of_string Yojson.Safe.read_json trustees in
+  let rec loop i ts =
+    match ts with
+    | [] -> Lwt.return_none (* an error, actually *)
+    | `Single _ :: ts -> loop (i - 1) ts
+    | `Pedersen _ :: _ -> Lwt.return_some (List.nth keys i)
+  in
+  loop (trustee_id - 1) trustees
 
 let with_tally_trustee token uuid f =
   let@ token = Option.unwrap unauthorized token in
@@ -158,7 +154,7 @@ let perform_server_side_decryption election metadata tally =
     | Some sk ->
        let* pd = W.E.compute_factor tally sk in
        let pd = string_of_partial_decryption W.G.write pd in
-       Web_persist.set_partial_decryptions uuid [i, pd]
+       Web_persist.add_partial_decryption uuid (i, pd)
     | None ->
        Lwt.fail (Error "missing server private key")
   in
@@ -278,7 +274,6 @@ let finish_shuffling election metadata =
 let release_tally election =
   let module W = (val election : Site_common_sig.ELECTION_LWT) in
   let uuid = W.election.e_uuid in
-  let uuid_s = raw_string_of_uuid uuid in
   let@ () = fun cont ->
     let* state = Web_persist.get_election_state uuid in
     match state with
@@ -329,30 +324,34 @@ let release_tally election =
   match W.E.compute_result ?shuffles ?shufflers ntallied et pds trustees with
   | Ok result ->
      let* () =
-       let result = string_of_election_result W.G.write W.write_result result in
+       let result =
+         string_of_election_result
+           W.write_result (write_encrypted_tally W.G.write)
+           (write_partial_decryption W.G.write) (write_shuffle W.G.write)
+           result
+       in
        write_file ~uuid (string_of_election_file ESResult) [result]
      in
      let* () = Web_persist.remove_audit_cache uuid in
      let* () = Web_persist.set_election_state uuid `Tallied in
      let* dates = Web_persist.get_election_dates uuid in
      let* () = Web_persist.set_election_dates uuid {dates with e_tally = Some (now ())} in
-     let* () = cleanup_file (!Web_config.spool_dir / uuid_s / "decryption_tokens.json") in
-     let* () = cleanup_file (!Web_config.spool_dir / uuid_s / "shuffles.jsons") in
+     let* () = cleanup_file (uuid /// "decryption_tokens.json") in
+     let* () = cleanup_file (uuid /// "shuffles.jsons") in
      let* () = Web_persist.clear_shuffle_token uuid in
      Lwt.return @@ Ok ()
   | Error e -> Lwt.return @@ Stdlib.Error (`CombinationError e)
 
 let delete_sensitive_data uuid =
-  let uuid_s = raw_string_of_uuid uuid in
-  let* () = cleanup_file (!Web_config.spool_dir / uuid_s / "state.json") in
-  let* () = cleanup_file (!Web_config.spool_dir / uuid_s / "decryption_tokens.json") in
-  let* () = cleanup_file (!Web_config.spool_dir / uuid_s / "partial_decryptions.json") in
-  let* () = cleanup_file (!Web_config.spool_dir / uuid_s / "extended_records.jsons") in
-  let* () = cleanup_file (!Web_config.spool_dir / uuid_s / "credential_mappings.jsons") in
-  let* () = rmdir (!Web_config.spool_dir / uuid_s / "ballots") in
-  let* () = cleanup_file (!Web_config.spool_dir / uuid_s / "ballots_index.json") in
-  let* () = cleanup_file (!Web_config.spool_dir / uuid_s / "private_key.json") in
-  let* () = cleanup_file (!Web_config.spool_dir / uuid_s / "private_keys.jsons") in
+  let* () = cleanup_file (uuid /// "state.json") in
+  let* () = cleanup_file (uuid /// "decryption_tokens.json") in
+  let* () = cleanup_file (uuid /// "partial_decryptions.json") in
+  let* () = cleanup_file (uuid /// "extended_records.jsons") in
+  let* () = cleanup_file (uuid /// "credential_mappings.jsons") in
+  let* () = rmdir (uuid /// "ballots") in
+  let* () = cleanup_file (uuid /// "ballots_index.json") in
+  let* () = cleanup_file (uuid /// "private_key.json") in
+  let* () = cleanup_file (uuid /// "private_keys.jsons") in
   Lwt.return_unit
 
 let archive_election uuid =
@@ -363,7 +362,6 @@ let archive_election uuid =
 let delete_election election metadata =
   let module W = (val election : Site_common_sig.ELECTION_LWT) in
   let uuid = W.election.e_uuid in
-  let uuid_s = raw_string_of_uuid uuid in
   let* () = delete_sensitive_data uuid in
   let de_template =
     {
@@ -412,18 +410,29 @@ let delete_election election metadata =
   let* voters = Web_persist.get_voters uuid in
   let* ballots = Web_persist.get_ballot_hashes uuid in
   let* result = Web_persist.get_election_result uuid in
+  let de_nb_voters, de_has_weights =
+    match voters with
+    | None -> 0, false
+    | Some voters ->
+       List.length voters,
+       List.exists
+         (fun x ->
+           let _, _, weight = split_identity_opt x in
+           weight <> None
+         ) voters
+  in
   let de = {
       de_uuid = uuid;
       de_template;
       de_owner;
-      de_nb_voters = (match voters with None -> 0 | Some x -> List.length x);
+      de_nb_voters;
       de_nb_ballots = List.length ballots;
       de_date;
       de_tallied = result <> None;
       de_authentication_method;
       de_credential_method;
       de_trustees;
-      de_server_is_trustee = metadata.e_server_is_trustee = Some true;
+      de_has_weights;
     }
   in
   let* () = write_file ~uuid "deleted.json" [string_of_deleted_election de] in
@@ -449,14 +458,13 @@ let delete_election election metadata =
   let* () =
     Lwt_list.iter_p
       (fun x ->
-        cleanup_file (!Web_config.spool_dir / uuid_s / x)
+        cleanup_file (uuid /// x)
       ) files_to_delete
   in
   Web_persist.clear_elections_by_owner_cache ()
 
 let find_user_id uuid user =
-  let uuid_s = raw_string_of_uuid uuid in
-  let db = Lwt_io.lines_of_file (!Web_config.spool_dir / uuid_s / "voters.txt") in
+  let db = Lwt_io.lines_of_file (uuid /// "voters.txt") in
   let* db = Lwt_stream.to_list db in
   let rec loop = function
     | [] -> None
@@ -474,8 +482,7 @@ let find_user_id uuid user =
   Lwt.return (loop db, show_weight)
 
 let load_password_db uuid =
-  let uuid_s = raw_string_of_uuid uuid in
-  let db = !Web_config.spool_dir / uuid_s / "passwords.csv" in
+  let db = uuid /// "passwords.csv" in
   Lwt_preemptive.detach Csv.load db
 
 let rec replace_password username ((salt, hashed) as p) = function
@@ -572,11 +579,9 @@ let get_trustee_names uuid =
   Lwt.return (extract_names trustees)
 
 let get_trustee_name uuid metadata trustee =
-  match metadata.e_trustees with
-  | None -> Lwt.return_none
-  | Some xs ->
-     let* names = get_trustee_names uuid in
-     Lwt.return (List.assoc trustee (List.combine xs names))
+  let&* xs = metadata.e_trustees in
+  let* names = get_trustee_names uuid in
+  Lwt.return (List.assoc trustee (List.combine xs names))
 
 let skip_shuffler uuid metadata trustee =
   let* sh_name = get_trustee_name uuid metadata trustee in
