@@ -84,10 +84,10 @@ module Make (Web_state : Web_state_sig.S) (Web_i18n : Web_i18n_sig.S) (Web_servi
         (fun l ->
           li [match l with
               | [] -> failwith "anomaly in Pages_voter.majority_judgment"
-              | [x] -> txt x
+              | [x] -> span (txt_br x)
               | l -> div [
                          txt (s_ "Tie:");
-                         ul (List.map (fun x -> li [txt x]) l);
+                         ul (List.map (fun x -> li (txt_br x)) l);
                        ]
             ]
         ) explicit_winners
@@ -140,10 +140,10 @@ module Make (Web_state : Web_state_sig.S) (Web_i18n : Web_i18n_sig.S) (Web_servi
         (fun l ->
           li [match l with
               | [] -> failwith "anomaly in Web_templates.schulze"
-              | [x] -> txt x
+              | [x] -> span (txt_br x)
               | l -> div [
                          txt (s_ "Tie:");
-                         ul (List.map (fun x -> li [txt x]) l);
+                         ul (List.map (fun x -> li (txt_br x)) l);
                        ]
             ]
         ) explicit_winners
@@ -180,7 +180,7 @@ module Make (Web_state : Web_state_sig.S) (Web_i18n : Web_i18n_sig.S) (Web_servi
        in
        let answers =
          List.mapi (fun j x ->
-             tr [td [txt x]; td [txt @@ Weight.to_string r.(j)]]
+             tr [td (txt_br x); td [txt @@ Weight.to_string r.(j)]]
            ) answers
        in
        let answers =
@@ -192,7 +192,7 @@ module Make (Web_state : Web_state_sig.S) (Web_i18n : Web_i18n_sig.S) (Web_servi
             | _ -> table (y :: ys)
        in
        li ~a:[a_class ["result_question_item"]] [
-           div ~a:[a_class ["result_question"]] [txt x.q_question];
+           div ~a:[a_class ["result_question"]] (txt_br x.q_question);
            answers;
          ]
     | Question.NonHomomorphic (q, extra), RNonHomomorphic ballots ->
@@ -231,7 +231,7 @@ module Make (Web_state : Web_state_sig.S) (Web_i18n : Web_i18n_sig.S) (Web_servi
          ) else txt ""
        in
        li ~a:[a_class ["result_question_item"]] [
-           div ~a:[a_class ["result_question"]] [txt q.q_question];
+           div ~a:[a_class ["result_question"]] (txt_br q.q_question);
            applied_counting_method;
            div [
                txt (s_ "The raw results can be viewed in the ");
@@ -343,7 +343,13 @@ module Make (Web_state : Web_state_sig.S) (Web_i18n : Web_i18n_sig.S) (Web_servi
     in
     let* middle =
       let* result = Web_persist.get_election_result uuid in
-      let result = Option.map (election_result_of_string W.G.read W.read_result) result in
+      let result =
+        Option.map (
+            election_result_of_string
+              W.read_result Yojson.Safe.read_json
+              Yojson.Safe.read_json Yojson.Safe.read_json
+          ) result
+      in
       let* hidden = Web_persist.get_election_result_hidden uuid in
       let* is_admin =
         let* metadata = Web_persist.get_election_metadata uuid in
@@ -422,17 +428,13 @@ module Make (Web_state : Web_state_sig.S) (Web_i18n : Web_i18n_sig.S) (Web_servi
         ]
     in
     let div_voters =
-      div [
+      div ~a:[a_id "voters"] [
           Printf.ksprintf txt
             (f_ "The voter list has %d voter(s) and fingerprint %s.")
-            cache.cache_num_voters cache.cache_voters_hash;
+            cache.cache_checksums.ec_num_voters (Hash.to_b64 cache.cache_voters_hash);
         ]
     in
-    let show_weights =
-      match cache.cache_total_weight with
-      | Some x when not Weight.(is_int x cache.cache_num_voters) -> true
-      | _ -> false
-    in
+    let show_weights = cache.cache_checksums.ec_weights <> None in
     let div_show_weights =
       if show_weights then
         div [
@@ -444,12 +446,14 @@ module Make (Web_state : Web_state_sig.S) (Web_i18n : Web_i18n_sig.S) (Web_servi
       else txt ""
     in
     let div_total_weight =
-      match cache.cache_total_weight, cache.cache_min_weight, cache.cache_max_weight with
-      | Some w, Some min, Some max ->
-         div [
+      match cache.cache_checksums.ec_weights with
+      | Some {w_total; w_min; w_max} ->
+         div ~a:[a_id "weights"] [
              Printf.ksprintf txt
                (f_ "The total weight is %s (min: %s, max: %s).")
-               (Weight.to_string w) (Weight.to_string min) (Weight.to_string max);
+               (Weight.to_string w_total)
+               (Weight.to_string w_min)
+               (Weight.to_string w_max);
            ]
       | _ -> txt ""
     in
@@ -458,7 +462,7 @@ module Make (Web_state : Web_state_sig.S) (Web_i18n : Web_i18n_sig.S) (Web_servi
           List.map
             (fun x ->
               let name = Option.value x.tc_name ~default:"N/A" in
-              li [Printf.ksprintf txt "%s (%s)" name x.tc_checksum]
+              li [Printf.ksprintf txt "%s (%s)" name (Hash.to_b64 x.tc_checksum)]
             ) xs
         )
     in
@@ -478,32 +482,29 @@ module Make (Web_state : Web_state_sig.S) (Web_i18n : Web_i18n_sig.S) (Web_servi
               let name = Option.value x.ttc_name ~default:"N/A" in
               li [
                   Printf.ksprintf txt "%s (%s) [%s]"
-                    name x.ttc_verification_key x.ttc_pki_key
+                    name (Hash.to_b64 x.ttc_verification_key) (Hash.to_b64 x.ttc_pki_key)
                 ]
             ) xs
         )
     in
     let divs_trustees_threshold =
-      match checksums.ec_trustees_threshold with
-      | None -> []
-      | Some l ->
-         List.map
-           (fun x ->
-             div [
-                 Printf.ksprintf txt
-                   (f_ "%d of the following %d trustees (verification keys) [public keys] are needed to decrypt the election result:")
-                   x.ts_threshold (List.length x.ts_trustees);
-                 format_ttc "trustees_threshold" x.ts_trustees;
-               ]
-           ) l
+      List.map
+        (fun x ->
+          div [
+              Printf.ksprintf txt
+                (f_ "%d of the following %d trustees (verification keys) [public keys] are needed to decrypt the election result:")
+                x.ts_threshold (List.length x.ts_trustees);
+              format_ttc "trustees_threshold" x.ts_trustees;
+            ]
+        ) checksums.ec_trustees_threshold
     in
     let div_trustees = div (div_trustees_mandatory :: divs_trustees_threshold) in
     let div_credentials =
-      div [
+      div ~a:[a_id "credentials"] [
           Printf.ksprintf txt
             (f_ "Credentials were generated and sent by %s and have fingerprint %s.")
             (Option.value params.e_credential_authority ~default:"N/A")
-            checksums.ec_public_credentials;
+            (Hash.to_b64 checksums.ec_public_credentials);
         ]
     in
     let div_shuffles =
@@ -519,10 +520,10 @@ module Make (Web_state : Web_state_sig.S) (Web_i18n : Web_i18n_sig.S) (Web_servi
       match checksums.ec_encrypted_tally with
       | None -> txt ""
       | Some x ->
-         div [
+         div ~a:[a_id "encrypted_tally"] [
              Printf.ksprintf txt
                (f_ "The fingerprint of the encrypted tally is %s.")
-               x
+               (Hash.to_b64 x)
            ]
     in
     let div_audit =
@@ -549,7 +550,9 @@ module Make (Web_state : Web_state_sig.S) (Web_i18n : Web_i18n_sig.S) (Web_servi
         script ~a:[a_src (static "home.js")] (txt "");
       ] in
     let* lang_box = lang_box (ContSiteElection uuid) in
-    responsive_base ~lang_box ~title:params.e_name ~content ~footer ~uuid ()
+    let title = br_truncate params.e_name in
+    let full_title = params.e_name in
+    responsive_base ~lang_box ~full_title ~title ~content ~footer ~uuid ()
 
   let cast_raw election () =
     let* l = get_preferred_gettext () in
@@ -630,7 +633,9 @@ module Make (Web_state : Web_state_sig.S) (Web_i18n : Web_i18n_sig.S) (Web_servi
         form_upload;
       ] in
     let* footer = audit_footer election in
-    responsive_base ~title:params.e_name ~content ~uuid ~footer ()
+    let title = br_truncate params.e_name in
+    let full_title = params.e_name in
+    responsive_base ~full_title ~title ~content ~uuid ~footer ()
 
   let cast_confirmation election hash () =
     let* l = get_preferred_gettext () in
@@ -718,8 +723,8 @@ module Make (Web_state : Web_state_sig.S) (Web_i18n : Web_i18n_sig.S) (Web_servi
                                 ] in
     let* div_weight =
       let* audit_cache = Web_persist.get_audit_cache uuid in
-      match audit_cache.cache_total_weight with
-      | Some x when not Weight.(is_int x audit_cache.cache_num_voters) ->
+      match audit_cache.cache_checksums.ec_weights with
+      | Some _ ->
          let* ballot = Eliom_reference.get Web_state.ballot in
          (match ballot with
           | Some ballot ->
@@ -739,7 +744,7 @@ module Make (Web_state : Web_state_sig.S) (Web_i18n : Web_i18n_sig.S) (Web_servi
         progress_responsive;
         p [
             txt (s_ "Your ballot for ");
-            em [txt name];
+            em (txt_br name);
             txt (s_ " has been received, but not recorded yet. ");
             txt (s_ "Your smart ballot tracker is ");
             b ~a:[a_id "ballot_tracker"] [
@@ -764,13 +769,16 @@ module Make (Web_state : Web_state_sig.S) (Web_i18n : Web_i18n_sig.S) (Web_servi
             txt ".";
           ];
       ] in
-    responsive_base ~title:name ~content ~uuid ()
+    let title = br_truncate name in
+    let full_title = name in
+    responsive_base ~full_title ~title ~content ~uuid ()
 
   let lost_ballot election () =
     let* l = get_preferred_gettext () in
     let open (val l) in
     let open (val election : Site_common_sig.ELECTION_LWT) in
-    let title = election.e_name in
+    let title = br_truncate election.e_name in
+    let full_title = election.e_name in
     let uuid = election.e_uuid in
     let* metadata = Web_persist.get_election_metadata uuid in
     let Booth service = fst Web_services.booths.(get_booth_index metadata.e_booth_version) in
@@ -794,7 +802,7 @@ module Make (Web_state : Web_state_sig.S) (Web_i18n : Web_i18n_sig.S) (Web_servi
           ];
       ]
     in
-    responsive_base ~title ~content ~uuid ()
+    responsive_base ~full_title ~title ~content ~uuid ()
 
   let cast_confirmed election ~result () =
     let* l = get_preferred_gettext () in
@@ -887,7 +895,7 @@ module Make (Web_state : Web_state_sig.S) (Web_i18n : Web_i18n_sig.S) (Web_servi
           ];
         p ([
               txt (s_ "Your ballot for ");
-              em [txt name];
+              em (txt_br name);
             ] @ result);
         p
           [a
@@ -895,7 +903,9 @@ module Make (Web_state : Web_state_sig.S) (Web_i18n : Web_i18n_sig.S) (Web_servi
              [txt (s_ "Go back to election")]
              (uuid, ())];
       ] in
-    responsive_base ~title:name ~content ~uuid ()
+    let title = br_truncate name in
+    let full_title = name in
+    responsive_base ~full_title ~title ~content ~uuid ()
 
   let pretty_ballots election =
     let* l = get_preferred_gettext () in
@@ -904,14 +914,16 @@ module Make (Web_state : Web_state_sig.S) (Web_i18n : Web_i18n_sig.S) (Web_servi
     let uuid = election.e_uuid in
     let* hashes = Web_persist.get_ballot_hashes uuid in
     let* result = Web_persist.get_election_result uuid in
-    let result = Option.map (election_result_of_string G.read read_result) result in
-    let* audit_cache = Web_persist.get_audit_cache uuid in
-    let show_weights =
-      match audit_cache.cache_total_weight with
-      | Some x when not Weight.(is_int x audit_cache.cache_num_voters) -> true
-      | _ -> false
+    let result =
+      Option.map (
+          election_result_of_string
+            read_result Yojson.Safe.read_json
+            Yojson.Safe.read_json Yojson.Safe.read_json
+        ) result
     in
-    let title = election.e_name ^ " — " ^ s_ "Accepted ballots" in
+    let* audit_cache = Web_persist.get_audit_cache uuid in
+    let show_weights = audit_cache.cache_checksums.ec_weights <> None in
+    let title = br_truncate election.e_name ^ " — " ^ s_ "Accepted ballots" in
     let nballots = ref 0 in
     let hashes = List.sort (fun (a, _) (b, _) -> compare_b64 a b) hashes in
     let ballots =
