@@ -41,8 +41,6 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
   open Web_services
   open Site_common
 
-  let ( / ) = Filename.concat
-
   module PString = String
 
   open Eliom_service
@@ -194,7 +192,7 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
            if SetEmailOtp.check ~address ~code then (
              let a = {a with account_email = address} in
              let* () = Accounts.update_account a in
-             let* () = Eliom_reference.unset Web_state.set_email_env in
+             let* () = Web_state.discard () in
              Redirection.send (Redirection admin)
            ) else (
              let* l = get_preferred_gettext () in
@@ -822,8 +820,7 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
       (fun uuid () ->
         let@ _ = with_draft_election_ro uuid in
         let* () = Api_drafts.set_downloaded uuid in
-        File.send ~content_type:"text/plain"
-          (!Web_config.spool_dir / raw_string_of_uuid uuid / "private_creds.txt")
+        File.send ~content_type:"text/plain" (uuid /// "private_creds.txt")
       )
 
   let () =
@@ -881,31 +878,29 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
                 match election with
                 | None -> fail_http `Not_found
                 | Some se ->
-                   match List.find_opt (fun x -> token = x.st_token) se.se_public_keys with
-                   | None -> return_none
-                   | Some t ->
-                      if t.st_public_key <> "" then
-                        let msg = s_ "A public key already existed, the key you've just uploaded has been ignored!" in
-                        let title = s_ "Error" in
-                        return_some (title, msg, 400)
-                      else
-                        let version = Option.value se.se_version ~default:0 in
-                        let module G = (val Group.of_string ~version se.se_group : GROUP) in
-                        let module Trustees = (val Trustees.get_by_version (Option.value se.se_version ~default:0)) in
-                        let pk = trustee_public_key_of_string G.read public_key in
-                        let module K = Trustees.MakeCombinator (G) in
-                        if not (K.check [`Single pk]) then
-                          let msg = s_ "Invalid public key!" in
-                          let title = s_ "Error" in
-                          return_some (title, msg, 400)
-                        else (
-                          (* we keep pk as a string because of G.t *)
-                          t.st_public_key <- public_key;
-                          let* () = Web_persist.set_draft_election uuid se in
-                          let msg = s_ "Your key has been received and checked!" in
-                          let title = s_ "Success" in
-                          return_some (title, msg, 200)
-                        )
+                   let&* t = List.find_opt (fun x -> token = x.st_token) se.se_public_keys in
+                   if t.st_public_key <> "" then
+                     let msg = s_ "A public key already existed, the key you've just uploaded has been ignored!" in
+                     let title = s_ "Error" in
+                     return_some (title, msg, 400)
+                   else
+                     let version = Option.value se.se_version ~default:0 in
+                     let module G = (val Group.of_string ~version se.se_group : GROUP) in
+                     let module Trustees = (val Trustees.get_by_version (Option.value se.se_version ~default:0)) in
+                     let pk = trustee_public_key_of_string G.read public_key in
+                     let module K = Trustees.MakeCombinator (G) in
+                     if not (K.check [`Single pk]) then
+                       let msg = s_ "Invalid public key!" in
+                       let title = s_ "Error" in
+                       return_some (title, msg, 400)
+                     else (
+                       (* we keep pk as a string because of G.t *)
+                       t.st_public_key <- public_key;
+                       let* () = Web_persist.set_draft_election uuid se in
+                       let msg = s_ "Your key has been received and checked!" in
+                       let title = s_ "Success" in
+                       return_some (title, msg, 200)
+                     )
               )
           in
           match x with
@@ -1237,7 +1232,12 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
             match result with
             | None -> fail_http `Not_found
             | Some result ->
-               let result = election_result_of_string Yojson.Safe.read_json Yojson.Safe.read_json result in
+               let result =
+                 election_result_of_string
+                   Yojson.Safe.read_json Yojson.Safe.read_json
+                   Yojson.Safe.read_json Yojson.Safe.read_json
+                   result
+               in
                match result.result with
                | `List xs ->
                   (match List.nth_opt xs index with
@@ -1266,14 +1266,14 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
           let temp_dir = Filename.temp_file "belenios" "archive" in
           Sys.remove temp_dir;
           Unix.mkdir temp_dir 0o700;
-          Unix.mkdir (temp_dir / "public") 0o755;
-          Unix.mkdir (temp_dir / "restricted") 0o700;
+          Unix.mkdir (temp_dir // "public") 0o755;
+          Unix.mkdir (temp_dir // "restricted") 0o700;
           temp_dir
         ) ()
     in
     let* () =
       Lwt_list.iter_p (fun x ->
-          try_copy_file (!Web_config.spool_dir / uuid_s / x) (temp_dir / "public" / x)
+          try_copy_file (uuid /// x) (temp_dir // "public" // x)
         ) [
           "election.json";
           "trustees.json";
@@ -1284,7 +1284,7 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
     in
     let* () =
       Lwt_list.iter_p (fun x ->
-          try_copy_file (!Web_config.spool_dir / uuid_s / x) (temp_dir / "restricted" / x)
+          try_copy_file (uuid /// x) (temp_dir // "restricted" // x)
         ) [
           "voters.txt";
           "records";
@@ -1297,9 +1297,9 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
     let* r = Lwt_process.exec command in
     match r with
     | Unix.WEXITED 0 ->
-       let fname = !Web_config.spool_dir / uuid_s / "archive.zip" in
+       let fname = uuid /// "archive.zip" in
        let fname_new = fname ^ ".new" in
-       let* () = copy_file (temp_dir / "archive.zip") fname_new in
+       let* () = copy_file (temp_dir // "archive.zip") fname_new in
        let* () = Lwt_unix.rename fname_new fname in
        rmdir temp_dir
     | _ ->
@@ -1316,8 +1316,7 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
         let open (val l) in
         let* state = Web_persist.get_election_state uuid in
         if state = `Archived then (
-          let uuid_s = raw_string_of_uuid uuid in
-          let archive_name = !Web_config.spool_dir / uuid_s / "archive.zip" in
+          let archive_name = uuid /// "archive.zip" in
           let* b = file_exists archive_name in
           let* () = if not b then make_archive uuid else return_unit in
           File.send ~content_type:"application/zip" archive_name
@@ -1430,12 +1429,12 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
         in
         let pk = pks.(trustee_id-1).trustee_public_key in
         let pd = partial_decryption_of_string W.G.read partial_decryption in
-        let et = !Web_config.spool_dir / raw_string_of_uuid uuid / string_of_election_file ESETally in
+        let et = uuid /// string_of_election_file ESETally in
         let* et = Lwt_io.chars_of_file et |> Lwt_stream.to_string in
         let et = encrypted_tally_of_string W.G.read et in
-        if W.E.check_factor et pk pd then (
-          let pds = (trustee_id, partial_decryption) :: pds in
-          let* () = Web_persist.set_partial_decryptions uuid pds in
+        if string_of_partial_decryption W.G.write pd = partial_decryption && W.E.check_factor et pk pd then (
+          let pd = trustee_id, partial_decryption in
+          let* () = Web_persist.add_partial_decryption uuid pd in
           Pages_common.generic_page ~title:(s_ "Success")
             (s_ "Your partial decryption has been received and checked!") () >>=
             Html.send
@@ -1709,7 +1708,7 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
                    | _ -> failwith "Unknown step"
                  in
                  let* () =
-                   if Array.forall (fun x -> x.stt_step = Some 2) ts then (
+                   if Array.for_all (fun x -> x.stt_step = Some 2) ts then (
                      (try
                         K.step2 (get_certs ());
                         Array.iter (fun x -> x.stt_step <- Some 3) ts;
@@ -1719,7 +1718,7 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
                    ) else return_unit
                  in
                  let* () =
-                   if Array.forall (fun x -> x.stt_step = Some 4) ts then (
+                   if Array.for_all (fun x -> x.stt_step = Some 4) ts then (
                      (try
                         let certs = get_certs () in
                         let polynomials = get_polynomials () in
@@ -1734,7 +1733,7 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
                    ) else return_unit
                  in
                  let* () =
-                   if Array.forall (fun x -> x.stt_step = Some 6) ts then (
+                   if Array.for_all (fun x -> x.stt_step = Some 6) ts then (
                      (try
                         let certs = get_certs () in
                         let polynomials = get_polynomials () in
@@ -1894,8 +1893,7 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
              match x with
              | Ok () ->
                 let* () = Web_signup.remove_link email in
-                let* () = Eliom_reference.unset Web_state.signup_address in
-                let* () = Eliom_reference.unset Web_state.signup_env in
+                let* () = Web_state.discard () in
                 let service = preapply ~service:site_login (Some service, ContSiteAdmin) in
                 Pages_common.generic_page ~title:(s_ "Account creation") ~service (s_ "The account has been created.") ()
                 >>= Html.send
@@ -1919,8 +1917,7 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
              match x with
              | Ok () ->
                 let* () = Web_signup.remove_link address in
-                let* () = Eliom_reference.unset Web_state.signup_address in
-                let* () = Eliom_reference.unset Web_state.signup_env in
+                let* () = Web_state.discard () in
                 let service = preapply ~service:site_login (Some service, ContSiteAdmin) in
                 Pages_common.generic_page ~title:(s_ "Change password") ~service (s_ "The password has been changed.") ()
                 >>= Html.send
@@ -2003,35 +2000,31 @@ module Make (X : Pages_sig.S) (Site_common : Site_common_sig.S) (Web_auth : Web_
 
   let extract_automatic_data_draft uuid_s =
     let uuid = uuid_of_raw_string uuid_s in
-    let* election = Web_persist.get_draft_election uuid in
-    match election with
-    | None -> return_none
-    | Some se ->
-       let t = Option.value se.se_creation_date ~default:default_creation_date in
-       let next_t = datetime_add t (day days_to_delete) in
-       return_some (`Destroy, uuid, next_t)
+    let* se = Web_persist.get_draft_election uuid in
+    let&* se in
+    let t = Option.value se.se_creation_date ~default:default_creation_date in
+    let next_t = datetime_add t (day days_to_delete) in
+    return_some (`Destroy, uuid, next_t)
 
   let extract_automatic_data_validated uuid_s =
     let uuid = uuid_of_raw_string uuid_s in
     let* election = Web_persist.get_raw_election uuid in
-    match election with
-    | None -> return_none
-    | Some _ ->
-       let* state = Web_persist.get_election_state uuid in
-       let* dates = Web_persist.get_election_dates uuid in
-       match state with
-       | `Open | `Closed | `Shuffling | `EncryptedTally _ ->
-          let t = Option.value dates.e_finalization ~default:default_validation_date in
-          let next_t = datetime_add t (day days_to_delete) in
-          return_some (`Delete, uuid, next_t)
-       | `Tallied ->
-          let t = Option.value dates.e_tally ~default:default_tally_date in
-          let next_t = datetime_add t (day days_to_archive) in
-          return_some (`Archive, uuid, next_t)
-       | `Archived ->
-          let t = Option.value dates.e_archive ~default:default_archive_date in
-          let next_t = datetime_add t (day days_to_delete) in
-          return_some (`Delete, uuid, next_t)
+    let&* _ = election in
+    let* state = Web_persist.get_election_state uuid in
+    let* dates = Web_persist.get_election_dates uuid in
+    match state with
+    | `Open | `Closed | `Shuffling | `EncryptedTally _ ->
+       let t = Option.value dates.e_finalization ~default:default_validation_date in
+       let next_t = datetime_add t (day days_to_delete) in
+       return_some (`Delete, uuid, next_t)
+    | `Tallied ->
+       let t = Option.value dates.e_tally ~default:default_tally_date in
+       let next_t = datetime_add t (day days_to_archive) in
+       return_some (`Archive, uuid, next_t)
+    | `Archived ->
+       let t = Option.value dates.e_archive ~default:default_archive_date in
+       let next_t = datetime_add t (day days_to_delete) in
+       return_some (`Delete, uuid, next_t)
 
   let try_extract extract x =
     Lwt.catch
