@@ -11,7 +11,7 @@ from distutils.util import strtobool
 from selenium.common.exceptions import UnexpectedAlertPresentException
 from selenium.webdriver.support.select import Select
 from util.fake_sent_emails_manager import FakeSentEmailsManager
-from util.selenium_tools import wait_for_element_exists, wait_for_element_exists_and_contains_expected_text, wait_for_element_exists_and_has_non_empty_content, wait_for_an_element_with_partial_link_text_exists, set_element_attribute, wait_for_element_exists_and_has_non_empty_attribute, verify_all_elements_have_attribute_value, verify_some_elements_have_attribute_value, wait_for_an_element_with_link_text_exists
+from util.selenium_tools import wait_for_element_exists, wait_for_element_exists_and_contains_expected_text, wait_for_element_exists_and_has_non_empty_content, wait_for_an_element_with_partial_link_text_exists, set_element_attribute, wait_for_element_exists_and_has_non_empty_attribute, verify_all_elements_have_attribute_value, verify_some_elements_have_attribute_value, wait_for_elements_exist_and_are_visible
 from util.election_testing import random_email_addresses_generator, remove_database_folder, remove_election_from_database, wait_a_bit, build_css_selector_to_find_buttons_in_page_content_by_value, initialize_server, initialize_browser, election_page_url_to_election_id, verify_election_consistency, create_election_data_snapshot, delete_election_data_snapshot, log_in_as_administrator, log_out, administrator_starts_creation_of_election, administrator_edits_election_questions, administrator_sets_election_voters, administrator_validates_creation_of_election
 from util.election_test_base import BeleniosElectionTestBase
 from util.execution import console_log, ConsoleLogDuration
@@ -95,7 +95,7 @@ class BeleniosTestElectionScenario2Base(BeleniosElectionTestBase):
             subprocess.run(["rm", "-f", el]) # TODO: Execute a command that works on other OS, like `os.remove()`
 
 
-    def administrator_starts_creation_of_manual_election(self):
+    def administrator_starts_creation_of_manual_election(self, nh_question=False):
         # # Setting up a new election (action of the administrator)
 
         browser = self.browser
@@ -123,7 +123,7 @@ class BeleniosTestElectionScenario2Base(BeleniosElectionTestBase):
         # - She arrives on the Questions page. She checks that the page title is correct
         # - She removes answer 3
         # - She clicks on the "Save changes" button (this redirects to the "Preparation of election" page)
-        administrator_edits_election_questions(browser)
+        administrator_edits_election_questions(browser, nh_question)
 
         # She sets election's voters:
         # - She clicks on the "Edit voters" link, to then type the list of voters
@@ -165,7 +165,7 @@ class BeleniosTestElectionScenario2Base(BeleniosElectionTestBase):
         credential_authority_css_selector = "#main form input[name=__co_eliom_name]"
         credential_authority_element = wait_for_element_exists(browser, credential_authority_css_selector, settings.EXPLICIT_WAIT_TIMEOUT)
         credential_authority_element.clear()
-        credential_authority_element.send_keys("Cecily");
+        credential_authority_element.send_keys("Cecily")
         credential_authority_set_css_selector = "#main form input[type=submit]"
         credential_authority_set_element = browser.find_element_by_css_selector(credential_authority_set_css_selector)
         credential_authority_set_element.click()
@@ -214,15 +214,15 @@ class BeleniosTestElectionScenario2Base(BeleniosElectionTestBase):
         generate_button_element = wait_for_element_exists(browser, generate_button_css_selector)
         generate_button_element.click()
 
-        wait_a_bit()
+        wait_for_elements_exist_and_are_visible(browser, "#creds")
 
-        # She clicks on the "private credentials" and "public credentials" links and downloads these files. Files are by default downloaded to /tmp using filenames `creds.txt` and `public_creds.txt` respectively, but we choose to name them using an unique identifier instead.
+        # She clicks on the "private credentials" link and downloads this file. File is by default downloaded to /tmp using filename `creds.txt`, but we choose to name it using an unique identifier instead.
         link_css_ids = ["creds"]
         file_labels = ["private credentials"]
         link_css_selectors = ["#" + el for el in link_css_ids]
         for idx, link_css_id in enumerate(link_css_ids):
             link_element = wait_for_element_exists(browser, link_css_selectors[idx])
-            target_filename = str(uuid4())
+            target_filename = str(uuid4()) + ".txt"
             set_element_attribute(browser, link_css_id, 'download', target_filename)
             link_element.click()
             file_absolute_path = os.path.join(settings.BROWSER_DOWNLOAD_FOLDER, target_filename)
@@ -381,6 +381,11 @@ The election administrator.\
             browser = self.browser
             browser.get(link_for_this_trustee)
 
+            # He waits for the "Generate a key" button
+            generate_button_css_selector = "#interactivity button"
+            generate_button_expected_label = "Generate a key"
+            generate_button_element = wait_for_element_exists_and_contains_expected_text(browser, generate_button_css_selector, generate_button_expected_label)
+
             # He checks that the page content shows the same election URL as the one the administrator saw
             election_url_css_selector = "#main ul li"
             election_url_element = wait_for_element_exists_and_has_non_empty_content(browser, election_url_css_selector)
@@ -388,9 +393,6 @@ The election administrator.\
             assert election_url_content == self.election_page_url
 
             # He clicks on the "Generate a key" button
-            generate_button_css_selector = "#interactivity button"
-            generate_button_expected_label = "Generate a key"
-            generate_button_element = wait_for_element_exists_and_contains_expected_text(browser, generate_button_css_selector, generate_button_expected_label)
             generate_button_element.click()
 
             # He clicks on the "private key" and "public key" links, to download the private key and the public key (files are respectively saved by default as `private_key.json` and `public_key.json`, but we decide to save them as a unique file name)
@@ -398,7 +400,7 @@ The election administrator.\
             link_expected_labels = ["private key"]
             self.downloaded_files_paths_per_trustee[trustee_email_address] = dict()
             for idx2, link_css_id in enumerate(link_css_ids):
-                link_target_filename = str(uuid4())
+                link_target_filename = str(uuid4()) + ".json"
                 set_element_attribute(browser, link_css_id, 'download', link_target_filename)
                 link_expected_label = link_expected_labels[idx2]
                 link_element = wait_for_an_element_with_partial_link_text_exists(browser, link_expected_label)
@@ -466,9 +468,7 @@ The election administrator.\
 
         # She clicks on "en" language
         select = Select(wait_for_element_exists(browser, ".lang_box select", settings.EXPLICIT_WAIT_TIMEOUT))
-        select.select_by_visible_text("en")
-        submit = wait_for_element_exists(browser, ".lang_box input[type=submit]", settings.EXPLICIT_WAIT_TIMEOUT)
-        submit.click()
+        select.select_by_value("en")
 
         wait_a_bit()
 
@@ -629,9 +629,7 @@ The election administrator.\
 
         # She clicks on "en" language
         select = Select(wait_for_element_exists(browser, ".lang_box select", settings.EXPLICIT_WAIT_TIMEOUT))
-        select.select_by_visible_text("en")
-        submit = wait_for_element_exists(browser, ".lang_box input[type=submit]", settings.EXPLICIT_WAIT_TIMEOUT)
-        submit.click()
+        select.select_by_value("en")
 
         wait_a_bit()
 
@@ -758,6 +756,13 @@ if __name__ == "__main__":
     settings.CREDENTIAL_AUTHORITY_EMAIL_ADDRESS = os.getenv('CREDENTIAL_AUTHORITY_EMAIL_ADDRESS', settings.CREDENTIAL_AUTHORITY_EMAIL_ADDRESS)
     # TODO: settings.TRUSTEES_EMAIL_ADDRESSES (it cannot be manipulated the same way because it is an array)
 
+    if os.getenv('BOOTH_VERSION', None):
+        input_booth_version = os.getenv('BOOTH_VERSION')
+        if hasattr(settings.BOOTH_VERSIONS, input_booth_version):
+            settings.BOOTH_VERSION = getattr(settings.BOOTH_VERSION, input_booth_version)
+        else:
+            raise Exception("Error: Unknown value for BOOTH_VERSION:", input_booth_version)
+
     console_log("USE_HEADLESS_BROWSER:", settings.USE_HEADLESS_BROWSER)
     console_log("SENT_EMAILS_TEXT_FILE_ABSOLUTE_PATH:", settings.SENT_EMAILS_TEXT_FILE_ABSOLUTE_PATH)
     console_log("WAIT_TIME_BETWEEN_EACH_STEP:", settings.WAIT_TIME_BETWEEN_EACH_STEP)
@@ -776,4 +781,6 @@ if __name__ == "__main__":
     console_log("CREDENTIAL_AUTHORITY_EMAIL_ADDRESS:", settings.CREDENTIAL_AUTHORITY_EMAIL_ADDRESS)
     console_log("TRUSTEES_EMAIL_ADDRESSES:", settings.TRUSTEES_EMAIL_ADDRESSES)
 
+    console_log("BOOTH_VERSION:", settings.BOOTH_VERSION)
+
     unittest.main()
