@@ -202,7 +202,7 @@ The election administrator.\
             if trustee_email_address not in self.downloaded_files_paths_per_trustee:
                 self.downloaded_files_paths_per_trustee[trustee_email_address] = dict()
             for idx2, link_css_id in enumerate(link_css_ids):
-                link_target_filename = str(uuid4())
+                link_target_filename = str(uuid4()) + ".txt"
                 set_element_attribute(browser, link_css_id, 'download', link_target_filename)
                 link_expected_label = link_expected_labels[idx2]
                 link_element = wait_for_an_element_with_partial_link_text_exists(browser, link_expected_label)
@@ -601,6 +601,13 @@ if __name__ == "__main__":
     # TODO: settings.TRUSTEES_EMAIL_ADDRESSES (it cannot be manipulated the same way because it is an array)
     settings.TRUSTEES_THRESHOLD_VALUE = os.getenv('TRUSTEES_THRESHOLD_VALUE', settings.TRUSTEES_THRESHOLD_VALUE)
 
+    if os.getenv('BOOTH_VERSION', None):
+        input_booth_version = os.getenv('BOOTH_VERSION')
+        if hasattr(settings.BOOTH_VERSIONS, input_booth_version):
+            settings.BOOTH_VERSION = getattr(settings.BOOTH_VERSION, input_booth_version)
+        else:
+            raise Exception("Error: Unknown value for BOOTH_VERSION:", input_booth_version)
+
     console_log("USE_HEADLESS_BROWSER:", settings.USE_HEADLESS_BROWSER)
     console_log("SENT_EMAILS_TEXT_FILE_ABSOLUTE_PATH:", settings.SENT_EMAILS_TEXT_FILE_ABSOLUTE_PATH)
     console_log("WAIT_TIME_BETWEEN_EACH_STEP:", settings.WAIT_TIME_BETWEEN_EACH_STEP)
@@ -621,4 +628,6 @@ if __name__ == "__main__":
     console_log("NUMBER_OF_TRUSTEES:", settings.NUMBER_OF_TRUSTEES)
     console_log("TRUSTEES_THRESHOLD_VALUE:", settings.TRUSTEES_THRESHOLD_VALUE)
 
+    console_log("BOOTH_VERSION:", settings.BOOTH_VERSION)
+
     unittest.main()
