@@ -10,7 +10,8 @@ import re
 import json
 from selenium import webdriver
 from selenium.webdriver.common.keys import Keys
-from util.selenium_tools import wait_for_element_exists, wait_for_element_exists_and_contains_expected_text, wait_for_an_element_with_partial_link_text_exists, verify_element_label, wait_for_element_exists_and_attribute_contains_expected_text
+from selenium.common.exceptions import NoSuchElementException
+from util.selenium_tools import wait_for_element_exists, wait_for_element_exists_and_contains_expected_text, wait_for_an_element_with_partial_link_text_exists, verify_element_label, wait_for_element_exists_and_attribute_contains_expected_text, wait_until_page_url_changes
 from util.execution import console_log
 import settings
 
@@ -300,9 +301,9 @@ def verify_election_consistency(election_id, snapshot_folder=None):
 
     election_folder = os.path.join(settings.GIT_REPOSITORY_ABSOLUTE_PATH, settings.DATABASE_FOLDER_PATH_RELATIVE_TO_GIT_REPOSITORY, election_id)
     verification_tool_path = os.path.join(settings.GIT_REPOSITORY_ABSOLUTE_PATH, "_run/tool-debug/bin/belenios-tool")
-    command = [verification_tool_path, "verify"]
+    command = [verification_tool_path, "election", "verify"]
     if snapshot_folder:
-        command = [verification_tool_path, "verify-diff", "--dir1=" + snapshot_folder, "--dir2=" + election_folder]
+        command = [verification_tool_path, "election", "verify-diff", "--dir1=" + snapshot_folder, "--dir2=" + election_folder]
     running_process = subprocess.Popen(command, cwd=election_folder, stdout=subprocess.PIPE, stderr=subprocess.PIPE, universal_newlines=True)
     process_timeout = 15 # seconds
     try:
@@ -319,11 +320,11 @@ def verify_election_consistency(election_id, snapshot_folder=None):
         raise Exception("Error: Verification took longer than " + process_timeout + " seconds. STDOUT was: " + outs + " STDERR was:" + errs)
 
 
-def belenios_tool_generate_credentials(election_id, number_of_voters=None):
+def belenios_tool_generate_credentials(election_id, number_of_voters=None, nh_question=False):
     """
     Use local CLI belenios-tool to generate a number of credentials corresponding to the number of voters. Example:
     ```
-    ./_run/tool-debug/bin/belenios-tool credgen --uuid dmGuNVL1meanZt --group ./files/groups/default.json --count 5
+    ./_run/tool-debug/bin/belenios-tool setup generate-credentials --uuid dmGuNVL1meanZt --group ./files/groups/default.json --count 5
     5 private credentials with ids saved to ./1579802689.privcreds
     5 public credentials saved to ./1579802689.pubcreds
     5 hashed public credentials with ids saved to ./1579802689.hashcreds
@@ -334,8 +335,12 @@ def belenios_tool_generate_credentials(election_id, number_of_voters=None):
         number_of_voters = settings.NUMBER_OF_INVITED_VOTERS
     generated_files_destination_folder = settings.GENERATED_FILES_DESTINATION_FOLDER
     belenios_tool_path = os.path.join(settings.GIT_REPOSITORY_ABSOLUTE_PATH, "_run/tool-debug/bin/belenios-tool")
-    crypto_group_path = os.path.join(settings.GIT_REPOSITORY_ABSOLUTE_PATH, "files/groups/default.json")
-    command = [belenios_tool_path, "credgen", "--uuid", election_id, "--group", crypto_group_path, "--count", str(number_of_voters)]
+    if nh_question:
+        group_base_path = "files/groups/rfc3526-2048.json"
+    else:
+        group_base_path = "files/groups/default.json"
+    crypto_group_path = os.path.join(settings.GIT_REPOSITORY_ABSOLUTE_PATH, group_base_path)
+    command = [belenios_tool_path, "setup", "generate-credentials", "--uuid", election_id, "--group", crypto_group_path, "--count", str(number_of_voters)]
     running_process = subprocess.Popen(command, cwd=generated_files_destination_folder, stdout=subprocess.PIPE, stderr=subprocess.PIPE, universal_newlines=True)
     process_timeout = 15 * number_of_voters # seconds
     credential_file_id = None
@@ -386,7 +391,7 @@ def belenios_tool_generate_ballots(voters_data, global_credential_file_id, vote_
 
         # Execute belenios-tool to generate a vote ballot for voter
         voter_crypted_ballot_file = "voter_row_" + str(i) + "_crypted_ballot.json"
-        command = [belenios_tool_path, "vote", "--url", vote_page_url, "--privcred", voter_credential_file, "--ballot", voter_uncrypted_ballot_file, ">", voter_crypted_ballot_file]
+        command = [belenios_tool_path, "election", "generate-ballot", "--url", vote_page_url, "--privcred", voter_credential_file, "--ballot", voter_uncrypted_ballot_file, ">", voter_crypted_ballot_file]
         running_process = subprocess.Popen(" ".join(command), cwd=generated_files_destination_folder, stdout=subprocess.PIPE, stderr=subprocess.PIPE, universal_newlines=True, shell=True)
         process_timeout = 120 # seconds
         try:
@@ -415,7 +420,7 @@ def create_election_data_snapshot(election_id):
     else:
         raise Exception("Could not extract absolute path from output of mktemp:", out)
 
-    # Remark: If this command is run before any vote is cast, files `public_creds.txt` and `ballots.jsons` do not exist yet
+    # Remark: If this command is run before any vote is cast, file `ballots.jsons` does not exist yet
     subprocess.run(["cp", "election.json", "public_creds.txt", "trustees.json", "ballots.jsons", temporary_folder_absolute_path], cwd=election_folder) # TODO: Execute a command that works on other OS, like `shutil.copy()`
 
     return temporary_folder_absolute_path
@@ -425,6 +430,19 @@ def delete_election_data_snapshot(snapshot_folder):
     subprocess.run(["rm", "-rf", snapshot_folder]) # TODO: Execute a command that works on other OS, like `shutil.rmtree()`
 
 
+def accept_data_policy_if_present(browser):
+    # If a personal data policy modal appears (it does not appear after it has been accepted), she clicks on the "Accept" button
+    accept_button_label = "Accept"
+    try:
+        button_elements = find_buttons_in_page_content_by_value(browser, accept_button_label)
+        if len(button_elements) == 1:
+            button_elements[0].click()
+            return True
+    except NoSuchElementException:
+        pass
+    return False
+
+
 def log_in_as_administrator(browser, from_a_login_page=False):
     if from_a_login_page:
         local_login_link_label = settings.LOGIN_MODE
@@ -440,14 +458,7 @@ def log_in_as_administrator(browser, from_a_login_page=False):
 
         # She notices the page title mentions an election
         # TODO: Should we wait for the page to load here? It looks like we don't need to.
-        assert 'Election server' in browser.title, "Browser title was: " + browser.title
-
-        # If a personal data policy modal appears (it does not appear after it has been accepted), she clicks on the "Accept" button
-        accept_button_label = "Accept"
-        button_elements = find_buttons_in_page_content_by_value(browser, accept_button_label)
-        if len(button_elements) > 0:
-            assert len(button_elements) is 1
-            button_elements[0].click()
+        assert 'Belenios' in browser.title, "Browser title was: " + browser.title
 
         # She clicks on "local" to go to the login page
         login_link_css_selector = "#login_" + settings.LOGIN_MODE
@@ -471,6 +482,8 @@ def log_in_as_administrator(browser, from_a_login_page=False):
 
     wait_a_bit()
 
+    old_url = browser.current_url
+
     login_form_password_element.submit()
 
     # She verifies that she arrived on the administration page (instead of any login error page)
@@ -479,13 +492,15 @@ def log_in_as_administrator(browser, from_a_login_page=False):
     # - Sometimes we get an error like `selenium.common.exceptions.StaleElementReferenceException: Message: The element reference of <h1> is stale; either the element is no longer attached to the DOM, it is not in the current frame context, or the document has been refreshed` or `selenium.common.exceptions.NoSuchElementException: Message: Unable to locate element: #header h1`. This is because page content changed in between two of our instructions.
     # - Value read from the page is still the value contained in previous page, because page content has not changed yet.
 
+    wait_until_page_url_changes(browser, old_url, settings.EXPLICIT_WAIT_TIMEOUT)
+    accept_data_policy_if_present(browser)
     page_title_css_selector = "#header h1"
     page_title_expected_content = "Administration"
     wait_for_element_exists_and_contains_expected_text(browser, page_title_css_selector, page_title_expected_content, settings.EXPLICIT_WAIT_TIMEOUT)
 
 
 def election_home_find_start_button(browser):
-    return wait_for_element_exists_and_attribute_contains_expected_text(browser, "#main button", "onclick", "location.href='../../vote.html#uuid=", settings.EXPLICIT_WAIT_TIMEOUT)
+    return wait_for_element_exists(browser, "#start", settings.EXPLICIT_WAIT_TIMEOUT)
 
 
 def log_out(browser, election_id=None):
@@ -642,13 +657,29 @@ def administrator_edits_election_questions(browser, nh_question=False):
         nhtally_css_selector = ".nonhomomorphic_tally"
         nhtally_checkbox_element = browser.find_element_by_css_selector(nhtally_css_selector)
         nhtally_checkbox_element.click()
+    else:
+        # She sets max to 2
+        max_input = browser.find_element_by_css_selector(".question_max")
+        max_input.send_keys(Keys.BACKSPACE)
+        max_input.send_keys("2")
 
     # She removes answer 3
-    question_to_remove = 3
-    remove_button_css_selector = ".question_answer_item:nth-child(" + str(question_to_remove) + ") .btn_remove"
+    candidate_to_remove = 3
+    remove_button_css_selector = ".question_answer_item:nth-child(" + str(candidate_to_remove) + ") .btn_remove"
     remove_button_element = browser.find_element_by_css_selector(remove_button_css_selector)
     remove_button_element.click()
 
+    if settings.BOOTH_VERSION == settings.BOOTH_VERSIONS.RESPONSIVE_BOOTH:
+        booth_index = 1
+    else:
+        booth_index = 0
+
+    # In the booth type section, she clicks on the Nth radio button, to select the appropriate booth
+    booth_version_radio_buttons = browser.find_elements_by_css_selector("input[type=radio][name=booth_version_radio]")
+    if not booth_version_radio_buttons or len(booth_version_radio_buttons) != 2:
+        raise Exception("Booth version should be selected among 2 radio buttons")
+    booth_version_radio_buttons[booth_index].click()
+
     wait_a_bit()
 
     # She clicks on the "Save changes" button (this redirects to the "Preparation of election" page)
